#############################################################################################
#####Code to run the forward iteration (MC simulations) of the model for ENP blue whale######
#############################################################################################

#Load required libraries----
library(Rcpp)
library(RcppArmadillo)
library(RcppProgress)

#Load forward iteration function (Rcpp)----
sourceCpp(file='Rcpp functions/BW_Forward_function.cpp')

#Set simulation scenario----
scenario<-1 #this will influence pattern of anthropogenic disturbance and environmental variability (see manuscript for detailed description of simulated scenarios)

#Set model parameters (see backward iteration file for details)----
locs<-36
feed_g<-15
southloc<-10
aM<-6
N<-90
maxA<-N+aM
tb<-1
ti<-32
tw<-212
ds<-40
ns<-0:N
te1<-31
tbs<-tb+(aM +ns)*365
tis<-ti+(aM +ns)*365
tws<-tw+(aM +ns)*365
T<-tw+(aM+N)*365 
lb<-2500*0.94
ww<-17000+2500
lw<-ww*0.73
lgd<-(lw-lb)/211
leand<-c(lb,lb+cumsum(rep(lgd,211)))
ts<-(tw+1):T
lt<-22*(1-exp(-0.240*((ts/365)+4.5)))
leant<-0.73*4.6*(lt)^3.05
diff_l<-(leant[3]-leand[length(leand)])/3
leant[1:3]<-leand[length(leand)]+diff_l*(1:3)
lean_complete<-c(leand,leant)
leant_day<-round(lean_complete[2:length(lean_complete)]-lean_complete[1:(length(lean_complete)-1)])
per_lean<-seq(0.94,0.73,length.out=length(leand))
lt_prewean<-(leand/per_lean/4.6)^(1/3.05)
lt_complete<-c(lt_prewean,lt)
int_x<-2000
int_low_x<-500
max_perc<-rep(0.35,T)
max_perc[1:tw]<-0.44
max_perc[(tw+1):tbs[1]]<-seq(0.44,0.35,length.out=length((tw+1):tbs[1]))
min_perc<-0.16
death_perc<-0.05
min_xc<-0
max_C<-22000
xs_wide<-round(unique(c(seq(min_xc,min_xc+int_x,length.out=5),
           seq(min_xc+int_x,min_xc+int_x*2,length.out=5),
           seq(min_xc+int_x*2,min_xc+int_x*3,length.out=4),
           seq(min_xc+int_x*3,min_xc+int_x*4,length.out=3),
           seq(min_xc+int_x*4,min_xc+int_x*5,length.out=4),
           seq(min_xc+int_x*5,min_xc+int_x*6,length.out=3),
           seq(min_xc+int_x*5,max_C+int_x,by=int_x))),-2)
Ctmp<-max_perc*lean_complete/(1-max_perc)
x_lactmp<-min_perc*lean_complete/(1-min_perc)
xctmp<-death_perc*lean_complete/(1-death_perc)
C<-xs_wide[sapply(Ctmp,function(x) which.min(abs(x-xs_wide)))]
xc<-xs_wide[sapply(xctmp,function(x) which.min(abs(x-xs_wide)))]
x_lac<-xs_wide[sapply(x_lactmp,function(x) which.min(abs(x-xs_wide)))]
mc<-0.05
mC<-0.44
int_m<-0.03
ms<-seq(mc,mC+int_m,by=int_m)
for_hours<-read.csv("Ancillary files/Twilight_times_byLoc.csv",header=T)
for_hours<-rbind(for_hours[1,],for_hours)
for_hoursn<-t(for_hours)
for_hoursn2<-c(for_hoursn)
mdens<-rgamma(1000000,2.5,5.5)*0.8
mdens<-mdens[mdens<quantile(mdens,probs=0.999)]
if(scenario==7){mdens<-mdens*0.75}            
if(scenario==8){mdens<-mdens*0.95}
if(scenario==9){mdens<-mdens*0.85}
if(scenario==10){mdens<-mdens*0.975}
lag<-90
gcal_lag<-121+lag-74
upw<-read.csv("Ancillary files/Upwelling_Index.csv",header=T)
gcal<-data.frame(julday=0:365,loc=0,pr=0,se=0,pr_scaled=c(upw$pr_scaled[upw$loc==unique(upw$loc)[1]][gcal_lag:366],upw$pr_scaled[upw$loc==unique(upw$loc)[1]][1:(gcal_lag-1)]))
upw<-rbind(gcal,upw)
upw$pr_scaled[upw$pr_scaled<0]<-0
for (i in unique(upw$loc)[c(1,6:11)]){
 upw[upw$loc==i,]<-rbind(upw[upw$loc==i,][(366-lag+1):366,],upw[upw$loc==i,][1:(366-lag),])
 }
coords<-read.csv("Ancillary files/Locations_coordinates.csv",header=T)
coords<-rbind(c(0,0),coords)
yj<-array(0,dim=c(2,3,30,locs+1,366))
pj<-array(1/30,dim=c(2,3,30,locs+1,366))
rsize<-0.6
breaks<-cut(mdens*rsize,30)
yj[1,1,1:30,1:(feed_g-1),]<-tapply(mdens*rsize,breaks,mean)
pj[1,1,1:30,1:(feed_g-1),]<-tapply(mdens*rsize,breaks,length)/length(mdens)
breaks<-cut(mdens,30)
yj[1,2,1:30,feed_g:locs,]<-tapply(mdens,breaks,mean)
pj[1,2,1:30,feed_g:locs,]<-tapply(mdens,breaks,length)/length(mdens)
patch<-1
for (i in 1:locs){
 if(i>=feed_g){
  patch<-2
  }
 cl<-unique(upw$loc)[which.min(abs(coords[i,2]-unique(upw$loc)))]
 scaling<-upw$pr_scaled[upw$loc==cl]
 yj[1,patch,1:30,i,]<-t(t(yj[1,patch,1:30,i,])*scaling)
 }
breaks<-cut(mdens*rsize,30)
yj[2,1,1:30,1:(feed_g-1),]<-tapply(mdens*rsize,breaks,mean)
pj[2,1,1:30,1:(feed_g-1),]<-tapply(mdens*rsize,breaks,length)/length(mdens)
breaks<-cut(mdens,30)
yj[2,2,1:30,feed_g:locs,]<-tapply(mdens,breaks,mean)
pj[2,2,1:30,feed_g:locs,]<-tapply(mdens,breaks,length)/length(mdens)
for (i2 in 1:locs){
 if(i2>=feed_g & i2<=28){scal_Nino<-0.3}else{scal_Nino<-1}
 cl<-unique(upw$loc)[which.min(abs(coords[i2,2]-unique(upw$loc)))]
 scaling<-upw$pr_scaled[upw$loc==cl]
 yj[2,1,1:30,i2,]<-t(t(yj[2,1,1:30,i2,])*scaling*scal_Nino)
 yj[2,2,1:30,i2,]<-t(t(yj[2,2,1:30,i2,])*scaling*scal_Nino)
 }
pNo<- 0.2
yjn<-aperm(yj,c(2,1,3,4,5))
yjn2<-c(yjn)
pjn<-aperm(pj,c(2,1,3,4,5))
pjn2<-c(pjn)
lambda<-array(0,dim=c(2,locs+1,3,365))
lambda[1,1:(feed_g-1),1,]<-0.6
lambda[1,1:(feed_g-1),3,]<-0.4
lambda[1,feed_g:locs,2,]<-0.7
lambda[1,feed_g:locs,3,]<-0.3
lambda[1,locs+1,3,]<-1
maxNS<-28
red2<-0.3
lambdatmp1<-0.6
lambdatmp2<-0.7
lambda[2,1:(feed_g-1),1,]<-lambdatmp1
lambda[2,1:(feed_g-1),3,]<-1-lambdatmp1
lambda[2,feed_g:maxNS,2,]<-lambdatmp2*red2
lambda[2,feed_g:maxNS,3,]<-1-lambdatmp2
lambda[2,feed_g:maxNS,1,]<-1-(1-lambdatmp2)-lambdatmp2*red2
lambda[2,(maxNS+1):locs,2,]<-lambdatmp2
lambda[2,(maxNS+1):locs,3,]<-1-lambdatmp2
lambda[2,locs+1,3,]<-1
lambdan<-aperm(lambda,c(2,1,3,4))
lambdan2<-c(lambdan)
rm(lambda, lambdan, yj, yjn, pj, pjn, for_hours, for_hoursn)
gc()

#Forward simulation settings----
simY<- maxA+2-1             #number of years over which we want to simulate the females, + 2 so that size of arrays doesn't create issues, -1 because they are already 1 at start of simulation
simT<-365*simY              #corresponding number of days (to determine size of arrays)
ni<-1000                    #number of females to simulate
 tws_f<-tw+(0:(simY-1))*365 #time steps in forward simulation for weaning of calves

#Initialise arrays to store state variables and other info for each whale (ni*simT)----
x<-array(-1,dim=c(ni,simT))               #array for female blubber mass
load("Initial values/Initial_condition_fromSim1.RData")  #load possible initial conditions from preliminary runs of the simulation
x[1:ni,1]<-sample(initscond,ni,replace=T) #assign random initial condition to each female
xn<-aperm(x,c(2,1))
xn2<-c(xn)                                #vector for C++
state<-array(-1,dim=c(ni,simT))           #array for reproductive state (1:4, -1 because used as index in C++)
state[1:ni,1]<-0                          #they are all juveniles
staten<-aperm(state,c(2,1))
staten2<-c(staten)                        #vector for C++
rm(x, xn, staten)
gc()
l<-array(NA,dim=c(ni,simT))               #array for location
load("Initial values/Initial_locations_fromSim1.RData")  #load possible initial locations from preliminary runs of the simulation
l[1:ni,1]<-sample(initsl,ni,replace=T)+1  #assign random initial location (+1 because it comes from simulation in C++)
ln<-aperm(l,c(2,1))
ln2<-c(ln)                                #vector for C++
i<-array(NA,dim=c(ni,simT))               #array for patch type (1:3, -1 because used as index in C++)
i[1:ni,1]<-sample(c(1,2),ni,replace=T)    #initial patch type depends on location; only patch type 2 or 3 in those initial locations
inn<-aperm(i,c(2,1))
in2<-c(inn)                               #vector for C++
st<-array(NA,dim=c(ni,simT))              #array for movement strategy
stn<-aperm(st,c(2,1))
stn2<-c(stn)                              #vector for C++
rm(l, ln, i, inn, st, stn)
gc()
m<-array(0,dim=c(ni,simT))                #array for calf condition
mn<-aperm(m,c(2,1))
mn2<-c(mn)                                #vector for C++
rm(m, mn, state)
gc()
consec<-rep(0,ni)                         #vector to store temporarily how many consecutive days are spent in a patch
alive<-c(rep(1,ni))                       #vector to store whether individuals are still alive
death<-c(rep(2,ni))                       #vector to store individuals' causes of death (1 starved, 2 old age, 3 other - by default all old age, changed in simulation as they die)
store<-array(0,dim=c(50000,9))            #to store history of each calf that is born 
storen<-aperm(store,c(2,1))
storen2<-c(storen)                        #vector for C++
rm(store, storen)
gc()
nsto<-0                                   #initial value of counter for storage dataframe

#Environmental variability----
enviro_history<-runif(simY*10,0.98,1.02)  #history of the environment (year-to-year variation in productivity)
if(scenario%in%c(2,42:47)){
  ENV<-"EnvChange"
  ye<-seq(4,simY*10,by=4)  #frequency of unfavourable years increases 
}else{
  if(scenario==3){
    ENV<-"EnvChange"
    ye<-seq(4,simY*10,by=3)  #frequency of unfavourable years increases 
  }else{
    if(scenario==4){
      ENV<-"EnvChange"
      ye<-seq(4,simY*10,by=2)  #frequency of unfavourable years increases 
    }else{
      ENV<-"ENSO"
      ye<-seq(4,simY*10,by=5)  #unfavourable years
    }
  }
}
enviro_history[ye]<-enviro_history[ye]+1  #mark unfavourable years in environmental history
if (scenario==5){
  ENV<-"EnvChange"
  yb<-seq(10,simY*10,by=10) #additional unfavourable years
  enviro_history[yb]<-1.8
}
if (scenario==6){
  ENV<-"EnvChange"
  yb<-seq(10,simY*10,by=20) #additional unfavourable years
  enviro_history[yb]<-1.8
}
if (scenario%in%c(7:10)){   #conditions worsen overall (see krill densities above)
  ENV<-"EnvChange"
}
env_start<-sample(1:(length(enviro_history)-simY),ni,replace=T) #assign to each female a random starting year in environmental history

#Anthropogenic disturbance----
if (scenario%in%c(1:10)){
  dist_locs<-array(0,dim=c(locs,simT))  #array for disturbance - each location on each day of simulation has a certain proportion that is exposed to disturbance (default:0)
  severity<-1                           #define the effect of the disturbance on feeding efficiency when staying in a disturbed location to feed (severity=1: no effect; severity=0:efficieny reduced to 0)
}
if (scenario==11){
  dist_locs<-array(0,dim=c(locs,simT))
  dist_locs[,1460:1474]<-0.95
  severity<-0.05
}
if (scenario==12){
  dist_locs<-array(0,dim=c(locs,simT))
  dist_locs[,1460:1520]<-0.95
  severity<-0.05
}
if (scenario==13){
  dist_locs<-array(0,dim=c(locs,simT))
  dist_locs[,1460:1640]<-0.95
  severity<-0.05
}
if (scenario==14){
  dist_locs<-array(0,dim=c(locs,simT))
  dist_locs[,1460:1825]<-0.95
  severity<-0.05
}
if (scenario%in%c(15,46)){
  dist_locs<-array(0,dim=c(locs,simT)) 
  dist_locs[sample(1:locs,locs/2,replace=F),1460:1825]<-0.95 
  severity<-0.05
}
if (scenario==16){
  dist_locs<-array(0,dim=c(locs,simT))
  dist_locs[14:17,1460:1825]<-0.95
  severity<-0.05
}
if (scenario==17){
  dist_locs<-array(0,dim=c(locs,simT))
  dist_locs[sample(1:locs,locs/2,replace=F),1460:1825]<-0.5
  severity<-0.5
}
if (scenario==18){
  dist_locs<-array(0,dim=c(locs,simT))
  dist_locs[,730:744]<-0.95
  severity<-0.05
}
if (scenario%in%c(19,42)){
  dist_locs<-array(0,dim=c(locs,simT))
  dist_locs[,730:790]<-0.95
  severity<-0.05
}
if (scenario==20){
  dist_locs<-array(0,dim=c(locs,simT))
  dist_locs[,2920:2980]<-0.95
  severity<-0.05
}
if (scenario%in%c(21,44)){
  dist_locs<-array(0,dim=c(locs,simT))
  dist_locs[,2920:3100]<-0.95
  severity<-0.05
}
if (scenario==22){
  dist_locs<-array(0,dim=c(locs,simT))
  dist_locs[,730:1095]<-0.95
  severity<-0.05
}
if (scenario==23){
  dist_locs<-array(0,dim=c(locs,simT))
  dist_locs[,2920:3285]<-0.95
  severity<-0.05
}
if (scenario==24){
  dist_locs<-array(0,dim=c(locs,simT))
  dist_locs[,5475:5840]<-0.95
  severity<-0.05
}
if (scenario==25){
  dist_locs<-array(0,dim=c(locs,simT))
  distd<-rep((1:(simY-2))*365,each=60)+rep(1:60,simY-2)
  dist_locs[3:16,distd]<-0.95
  severity<-0.05
}
if (scenario==26){
  dist_locs<-array(0,dim=c(locs,simT))
  distd<-rep((1:(simY-2))*365,each=60)+rep(122:181,simY-2)
  dist_locs[3:16,distd]<-0.95
  severity<-0.05
}
if (scenario%in%c(27,45)){
  dist_locs<-array(0,dim=c(locs,simT))
  distd<-rep((1:(simY-2))*365,each=60)+rep(1:60,simY-2)
  dist_locs[,distd]<-0.95
  severity<-0.05
}
if (scenario==28){
  dist_locs<-array(0,dim=c(locs,simT))
  distd<-rep((1:(simY-2))*365,each=180)+rep(1:180,simY-2)
  dist_locs[,distd]<-0.95
  severity<-0.05
}
if (scenario==29){
  dist_locs<-array(0,dim=c(locs,simT))
  distd<-rep((1:(simY-2))*365,each=180)+rep(1:180,simY-2)
  dist_locs[sample(1:locs,locs/2,replace=F),distd]<-0.95
  severity<-0.05
}
if (scenario==30){
  dist_locs<-array(0,dim=c(locs,simT))
  distd<-rep((1:(simY-2))*365,each=185)+rep(181:365,simY-2)
  dist_locs[sample(1:locs,locs/2,replace=F),distd]<-0.95
  severity<-0.05
}
if (scenario==31){
  dist_locs<-array(0,dim=c(locs,simT))
  dist_locs[sample(1:locs,locs/2,replace=F),]<-0.25
  severity<-0.75
}
if (scenario==32){
  dist_locs<-array(0,dim=c(locs,simT))
  dist_locs[sample(1:locs,locs/2,replace=F),]<-0.5
  severity<-0.5
}
if (scenario==33){
  dist_locs<-array(0,dim=c(locs,simT))
  dist_locs[sample(1:locs,locs/3,replace=F),]<-0.5
  severity<-0.5
}
if (scenario==34){
  dist_locs<-array(0,dim=c(locs,simT))
  dist_locs[sample(1:locs,locs/4,replace=F),]<-0.5
  severity<-0.5
}
if (scenario==35){
  dist_locs<-array(0,dim=c(locs,simT))
  dist_locs[14:17,]<-0.5
  severity<-0.5
}
if (scenario==36){
  dist_locs<-array(0,dim=c(locs,simT))
  dist_locs[15:16,]<-0.5
  severity<-0.5
}
if (scenario==37){
  dist_locs<-array(0,dim=c(locs,simT))
  dist_locs[sample(1:locs,locs/2,replace=F),]<-0.5
  severity<-0.75
}
if (scenario==38){
  dist_locs<-array(0,dim=c(locs,simT))
  dist_locs[sample(1:locs,locs/2,replace=F),]<-0.25
  severity<-0.5
}
if (scenario==39){
  dist_locs<-array(0,dim=c(locs,simT))
  dist_locs[sample(1:locs,locs/2,replace=F),]<-0.95
  severity<-0.05
}
if (scenario==40){
  dist_locs<-array(0,dim=c(locs,simT))
  dist_locs[sample(1:locs,locs/2,replace=F),rep(seq(0,maxA,by=2)*365,each=365)+rep(1:365,length(seq(0,maxA,by=2)))]<-0.5
  severity<-0.5
}
if (scenario==41){
  dist_locs<-array(0,dim=c(locs,simT))
  dist_locs[sample(1:locs,locs/2,replace=F),rep(seq(0,maxA,by=3)*365,each=365)+rep(1:365,length(seq(0,maxA,by=3)))]<-0.5
  severity<-0.5
}
if (scenario==43){
  dist_locs<-array(0,dim=c(locs,simT))
  dist_locs[sample(1:locs,locs/2,replace=F),2920:3100]<-0.95
  severity<-0.05
}
if (scenario==47){
  dist_locs<-array(0,dim=c(locs,simT))
  dist_locs[sample(1:locs,locs/2,replace=F),]<-0.25
  severity<-0.75
}
dist_locsn<-aperm(dist_locs,c(2,1))
dist_locsn2<-c(dist_locsn)         #vector for C++
rm(dist_locs, dist_locsn)
gc()
exposed<-array(0,dim=c(ni,simT))   #array for exposure
exposedn<-aperm(exposed,c(2,1))
exposedn2<-c(exposedn)             #vector for C++
resp<-array(0,dim=c(ni,simT))      #array for responses to disturbance
respn<-aperm(resp,c(2,1))
respn2<-c(respn)                   #vector for C++
rm(resp, respn, exposed, exposedn)
gc()
responsive<-rep(0,ni)
responsive[1:ni]<-rbeta(ni,5,5)    #draw a level of responsiveness for each female from a beta distribution. 1=they are responsive and flee; 0=they are not responsive and stay

if (scenario%in%c(11:17,46)){      #when disturbance occurs in a random year, select which year it is
  start_dist<-sample(seq(0,simT-1,by=365),ni,replace=T)
}else{
  start_dist<-rep(0,ni)
}

#Run MC simulation----

for (y in 0:(simY-2)) {                                                     #cycle over years of the simulation 
  env<-round(enviro_history[env_start+y])-1                                 #environmental conditions (for each female, on that year)
  prod<-enviro_history[env_start+y]-round(enviro_history[env_start+y])+1    #environmental variability (scalar that multiplies krill density; one value per female on that year)
  agein<-c(rep(-1,ni))                                                      #vector for age of the calf at weaning
  calves<-c(rep(0,ni))                                                      #vector to store whether there is a calf for a given female to model (between weaning and age 1)
  x_calves<-array(-1,dim=c(ni,365))                                         #array for blubber mass of calves
  xn_calves<-aperm(x_calves,c(2,1))
  xn2_calves<-c(xn_calves)                                                  #vector for C++               
  l_calves<-array(-1,dim=c(ni,365))                                         #array for location of calves
  ln_calves<-aperm(l_calves,c(2,1))
  ln2_calves<-c(ln_calves)                                                  #vector for C++
  i_calves<-array(-1,dim=c(ni,365))                                         #array for patch type of calves
  inn_calves<-aperm(i_calves,c(2,1))
  in2_calves<-c(inn_calves)                                                 #vector for C++
  st_calves<-array(-1,dim=c(ni,365))                                        #array for strategy of calves
  stn_calves<-aperm(st_calves,c(2,1))
  stn2_calves<-c(stn_calves)                                                #vector for C++
  exposed_calves<-rep(0,ni)
  resp_calves<-rep(0,ni)                                                    #calves' response to disturbance
  responsive_calves<-rep(0,ni)                                              #calves' responsiveness
  consec_calves<-rep(0,ni)                                                  #monitor consecutive days a calf spends in a patch
  rm(x_calves,xn_calves,l_calves,ln_calves,i_calves,inn_calves,st_calves,stn_calves)
  gc()
  if (any(alive==1)){                        #if any female is alive at the start of that year
   for (dd in 1:365){                        #dd is the day of that year
    if(y==95 & dd==212){alive<-c(rep(0,ni))} #95th simulated year is 96th of life of a female, so on last tw (212) she dies
    nam2<-paste("Backward SDP results/ResultsSDP_strategyD_day",dd,".RData",sep="") #load behavioural strategy for that day
    load(nam2)
    resDl_dn<-aperm(resDl_d,c(2,1,3,4,5,6,7))
    resDl_dn2<-c(resDl_dn)                   #vector for C++
    nam3<-paste("Backward SDP results/ResultsSDP_strategyR_day",dd,".RData",sep="") #load reproductive strategy for that day
    load(nam3)
    resRl_dn<-aperm(resRl_d,c(2,1,3,4,5,6,7))
    resRl_dn2<-c(resRl_dn)                   #vector for C++
    rm(resDl_d,resDl_dn,resRl_d,resRl_dn)
    nam4<-paste("Backward SDP results/ResultsSDP_strategyD2_day",dd,".RData",sep="")#load alternative behavioural strategy for that day
    load(nam4)
    resD2l_dn<-aperm(resD2l_d,c(2,1,3,4,5,6,7))
    resD2l_dn2<-c(resD2l_dn)                 #vector for C++
    rm(resD2l_d,resD2l_dn)
    #Apply forward function:
    tmpf<-forward_day_d(yjn2,pjn2,for_hoursn2,lambdan2,xn2,staten2,ln2,in2,stn2,mn2,storen2,consec,consec_calves,alive,death,nsto,calves,agein,xn2_calves,ln2_calves,in2_calves,stn2_calves,resDl_dn2, resRl_dn2, resD2l_dn2,simY,y,dd, prod, env, dist_locsn2, responsive, responsive_calves, exposedn2, respn2, exposed_calves, resp_calves, severity, start_dist)
    #Update vectors with new results
    xn2<-tmpf$x
    staten2<-tmpf$state
    ln2<-tmpf$l
    in2<-tmpf$i
    stn2<-tmpf$st
    mn2<-tmpf$m
    storen2<-tmpf$store
    consec<-tmpf$consec
    consec_calves<-tmpf$consec_calves
    alive<-tmpf$alive
    death<-tmpf$death
    nsto<-tmpf$nsto
    calves<-tmpf$calves
    agein<-tmpf$agein
    xn2_calves<-tmpf$x_calves
    ln2_calves<-tmpf$l_calves
    in2_calves<-tmpf$i_calves
    stn2_calves<-tmpf$st_calves
    exposedn2<-tmpf$exposed
    respn2<-tmpf$resp
    exposed_calves<-tmpf$exposed_calves
    resp_calves<-tmpf$resp_calves
    responsive_calves<-tmpf$responsive_calves
    }
   }
  print(y) #keep track of MC simulation progress
  }

nam<-paste("Forward MC simulations/Results_ForwardIteration_scenario",scenario,"_env",ENV,".RData",sep="")
save.image(file=nam) #save simulation results



