#include <math.h>
// [[Rcpp::depends(RcppArmadillo)]]
#include <RcppArmadillo.h>
#include <RcppArmadilloExtensions/sample.h>
using namespace std;
using namespace Rcpp;

//Ancillary functions

IntegerVector which_equal(NumericVector x, float args){
  std::vector<int> ind(x.size());
  for (int i=0; i<ind.size(); ++i){
    if(x[i]==args) {ind[i]=1;}else{ind[i]=0;}
  }
  std::vector<int> out(ind);
  std::vector<int>::iterator it;
  int j = 0;
  it = std::find(ind.begin(), ind.end(), 1);
  while(it++ != ind.end()){
    out[j++] = it - ind.begin();
    it = std::find(it, ind.end(), 1);
  }
  out.resize(j);
  return wrap(out);
}

bool contains(NumericVector X, double z) {
  return std::find(X.begin(), X.end(), z)!=X.end();
}

int vecmax(NumericVector x) {    
  NumericVector::iterator it = std::max_element(x.begin(), x.end());
  return it - x.begin();
}

int vecmin(NumericVector x) {    
  NumericVector::iterator it = std::min_element(x.begin(), x.end());
  return it - x.begin();
}

int vec2max(NumericVector x) {
  NumericVector sorted = clone(x).sort();
  IntegerVector matched= match(sorted, x);
  return matched[matched.size()-2];
}

float roundTo2dec(float x)
{
  NumericVector vec(1);
  vec[0]=x;
  vec=round(vec,2);
  return vec[0];
}

float roundToThousandths(float x)
{
  x /=1000;
  return floor(x + 0.5) * 1000;
}

float roundToFiveHundreds(float x)
{
  x /=500;
  return floor(x + 0.5) * 500;
}

float roundToInteger(float x, int y)
{
  x /=y;
  return floor(x + 0.5) * y;
}

float roundToHundreds(float x)
{
  x /=100;
  return floor(x + 0.5) * 100;
}

NumericVector sequence(float minx, float maxx, float intx)
{
  int n=(maxx-minx)/intx;
  NumericVector x(n+1);
  double total=minx;
  x[0]=total;
  for(int i = 1; i < (n+1); ++i) {
    total = total+intx;
    x[i] = total;
  }
  return x;
}

NumericVector sequence_inv(float maxx, float minx, float intx)
{
  int n=round((maxx-minx)/intx);
  NumericVector x(n+1);
  double total=maxx;
  x[0]=total;
  for(int i = 1; i < (n+1); ++i) {
    total = total-intx;
    x[i] = total;
  }
  return x;
}

//Functions for state dynamics

float grow(float leant_dayt, float meat_cal, float u)  //cost of growth on a given day
{
  return leant_dayt*meat_cal/u;
}

float ag(int gtime, NumericVector Wc_d, float cost_kg, NumericVector propg, float Chg_tot)   //gestational costs given stage of pregnancy in days
{
  return Wc_d[gtime]*cost_kg + propg[gtime]*Chg_tot;
}

float cot(float blubber, float leant) //cost of transport given total weight
{
  float cost=7.79*pow(blubber + leant,-0.29);
  return cost;
}

float blub_perc(float milk, int t, float blub_per, float me, float Er, float u, int t_il_t, NumericVector lean_complete, NumericVector leant_day, int tw, float meat_cal)  //new calf condition (proportion of mass that is blubber), given milk received, previous condition, and time 
{
  int day=t-t_il_t;
  float bm=lean_complete[day]*blub_per/(1-blub_per); //current blubber mass, derived from %blubber and lean mass in current time step
  float Ed=milk*me;                                  //energy received by the calf, given amount of milk
  float ErT, MRj;
  MRj=2*(4*(pow(bm + lean_complete[day],0.75))*(24*60*60)/1000); //metabolic rate of the calf, given size
  ErT=MRj+leant_day[day]*meat_cal;                   //energy required on that day for maintenance and growth
  float S=Ed-ErT;                                    //balance of energy - it can be negative
  float D=S/u;                                       //kg of blubber that are deposited/used as the result of energy balance
  float nbm=D+bm;                                    //new blubber mass
  float perc=nbm/(lean_complete[day+1]+nbm);         //new condition 
  return perc;            
  
}

float extra_m(float nper, int t, int t_il_t, float mC, float E, NumericVector lean_complete)   //how much blubber mass the calf would receive above the maximum % blubber (so that it is 'given back' to the mother, since this won't be wasted into milk delivery)
{
  float pMax=lean_complete[t-t_il_t+1]*mC/(1-mC);      //maximum blubber mass the calf can have on the following day
  float extra;
  float nbm=lean_complete[t-t_il_t+1]*nper/(1-nper);   //blubber mass it would achieve based on the milk the mother can give
  if (nper>mC){
    extra=(nbm-pMax);
  }else{
    extra=0;
  }
  extra=extra/E;
  return extra; //extra mass (not delivered to the calf but retained by the female)
}

NumericVector lr(NumericVector dens, NumericVector w)   //lunging rate given krill density
{
  NumericVector wa(dens.size());
  LogicalVector lll=(dens<0.2);                 
  LogicalVector mmm=(dens>=0.2 & dens<=0.4);
  LogicalVector hhh=(dens>0.4);
  for (int i=0; i<dens.size(); i++){
    if(lll[i]) wa[i]=w[0];                 
    if(mmm[i]) wa[i]=w[1];
    if(hhh[i]) wa[i]=w[2];
  }
  return wa;
}

NumericVector tf(NumericVector dens, NumericVector w, float beta_t, float psi_t, int tc, int l_i, int t, float for_hoursu)      //time spent feeding (given patch density and anatomical limitations at given length)
{
  NumericVector ti(dens.size());
  NumericVector ni(dens.size());
  NumericVector lunge=lr(dens,w);
  NumericVector hours(dens.size());
  NumericVector frate(dens.size());
  for (int i=0; i<dens.size(); i++){
    frate[i]=lunge[i]*beta_t*dens[i];                      //max amount of food per hour
    if(frate[i]!=0) ti[i]=psi_t/frate[i];                  //time (hours) to fill forestomach (depends on density)
    if((ti[i]>tc) && (ti[i]!=0)) ni[i]=for_hoursu/ti[i];   //number of times it can fill the forestomach in a 12 hour period
    if((ti[i]<=tc) && (ti[i]!=0)) ni[i]=for_hoursu/tc;
    hours[i]=ti[i]*ni[i];
  }
  return hours;
}

NumericVector cf(float blubber, float leant, float raamr_t, NumericVector dens, NumericVector w, float beta_t, float psi_t, int tc, int l_i, int t, float for_hoursu) //cost of feeding 
{
  NumericVector cfeed(dens.size());
  NumericVector hours=tf(dens,w,beta_t,psi_t,tc,l_i,t,for_hoursu);
  for (int i=0; i<dens.size(); i++){
    cfeed[i]=hours[i]*60*60*raamr_t*(blubber+leant)/1000; 
  }
  return cfeed;
}

NumericVector cr(float blubber, float leant, NumericVector dens, NumericVector w, float beta_t, float psi_t, int tc, int l_i, int t, float for_hoursu) //cost of resting (given patch density to calculate time spent foraging, weight and anatomical limitations at given length)
{
  float BMR=4*(pow(blubber + leant,0.75))*(24*60*60)/1000;  
  float BMRr=2*BMR;                                     
  NumericVector crest(dens.size());
  NumericVector hours=tf(dens,w,beta_t,psi_t,tc,l_i,t,for_hoursu);
  for (int i=0; i<dens.size(); i++){
    crest[i]=(24-hours[i])/24*BMRr;
  }
  return crest;
}

NumericVector food(NumericVector dens, NumericVector w, float beta_t, float psi_t, int tc, int l_i, int t, float for_hoursu, int rho, float A)  //amount of energy from engulfed food (given patch density and anatomical limitations at given length)
{
  NumericVector ti(dens.size());
  NumericVector ni(dens.size());
  NumericVector lunge=lr(dens,w);
  NumericVector fq(dens.size());
  NumericVector frate(dens.size());
  for (int i=0; i<dens.size(); i++){
    frate[i]=lunge[i]*beta_t*dens[i];                      //max amount of food per hour
    if(frate[i]!=0) ti[i]=psi_t/frate[i];                  //time (hours) to fill forestomach (depends on density)
    if((ti[i]>tc) && (ti[i]!=0)) ni[i]=for_hoursu/ti[i];   //number of times it can fill the forestomach in a 12 hour period
    if((ti[i]<=tc) && (ti[i]!=0)) ni[i]=for_hoursu/tc;
    fq[i]=ni[i]*psi_t*rho*A;
  }
  return fq;
}

//Functions for fitness interpolations

int ClosestLower(NumericVector xs_wide, float x)  //function to find closest lower bin
{
  NumericVector maxs=xs_wide[xs_wide <= x];
  IntegerVector cl= which_equal(xs_wide, max(maxs));
  return cl[0]-1; //-1 because used as index in C++
}


int Closest(NumericVector xs_wide, float x)  //function to find closest bin
{
  NumericVector diffs=abs(x-xs_wide);
  IntegerVector cl= which_equal(xs_wide,xs_wide[vecmin(diffs)]);
  return cl[0]-1; //-1 because used as index in C++
}

NumericVector interp_mx(NumericVector x1, NumericVector m1, float mc, NumericVector xs_wide, NumericVector ms, int int_x, float int_m, int l_i, int i,NumericVector F1,int locs, int nstates, int e_i)  //interpolation for state 4
{
  NumericVector delta_x(x1.size());
  NumericVector delta_m(m1.size());
  NumericVector interpolm(x1.size());
  for(int j=0; j<x1.size(); j++){
    int x1t_i=ClosestLower(xs_wide,x1[j]);
    delta_x[j]=(x1[j]-xs_wide[x1t_i])/(xs_wide[x1t_i+1]-xs_wide[x1t_i]);
    
    int m1t_i=(m1[j]-mc)/int_m;
    float mi=(ms.size()-1)/(max(ms)-mc)*(m1[j]-mc);
    delta_m[j]=mi-m1t_i;
    
    interpolm[j]=(1-delta_x[j])*(1-delta_m[j])*F1[e_i*xs_wide.size()+2*xs_wide.size()*m1t_i+2*xs_wide.size()*ms.size()*l_i+2*xs_wide.size()*ms.size()*(locs+1)*i+2*xs_wide.size()*ms.size()*(locs+1)*3*3 + x1t_i] +
      (1-delta_x[j])*delta_m[j]*F1[e_i*xs_wide.size()+2*xs_wide.size()*(m1t_i+1)+2*xs_wide.size()*ms.size()*l_i+2*xs_wide.size()*ms.size()*(locs+1)*i+2*xs_wide.size()*ms.size()*(locs+1)*3*3 + x1t_i]+
      delta_x[j]*(1-delta_m[j])*F1[e_i*xs_wide.size()+2*xs_wide.size()*m1t_i+2*xs_wide.size()*ms.size()*l_i+2*xs_wide.size()*ms.size()*(locs+1)*i+2*xs_wide.size()*ms.size()*(locs+1)*3*3 +  x1t_i+1] +
      delta_x[j]*delta_m[j]*F1[e_i*xs_wide.size()+2*xs_wide.size()*(m1t_i+1)+2*xs_wide.size()*ms.size()*l_i+2*xs_wide.size()*ms.size()*(locs+1)*i+2*xs_wide.size()*ms.size()*(locs+1)*3*3 +  x1t_i+1];
  }
  return interpolm;
}

NumericVector interp_x(NumericVector x1, NumericVector xs_wide, NumericVector ms, int int_x, int l_i, int i,NumericVector F1,int locs, int nstates, int state, int e_i)  //interpolation for other states
{
  NumericVector delta_x(x1.size());
  NumericVector interpolm(x1.size());
  int m_tok=9; //subtract 1 because C++
  for(int j=0; j<x1.size(); j++){
    int x1t_i=ClosestLower(xs_wide,x1[j]);
    delta_x[j]=(x1[j]-xs_wide[x1t_i])/(xs_wide[x1t_i+1]-xs_wide[x1t_i]);
    
    interpolm[j]=(1-delta_x[j])*F1[e_i*xs_wide.size()+2*xs_wide.size()*m_tok+2*xs_wide.size()*ms.size()*l_i+2*xs_wide.size()*ms.size()*(locs+1)*i+2*xs_wide.size()*ms.size()*(locs+1)*3*state + x1t_i] +
      delta_x[j]*F1[e_i*xs_wide.size()+2*xs_wide.size()*m_tok+2*xs_wide.size()*ms.size()*l_i+2*xs_wide.size()*ms.size()*(locs+1)*i+2*xs_wide.size()*ms.size()*(locs+1)*3*state + x1t_i + 1];
  }
  return interpolm;
}

int oneMultinom(Rcpp::NumericVector probs)
{
  int k = probs.size();
  Rcpp::IntegerVector ans(k);
  rmultinom(1, probs.begin(), k, ans.begin());
  for(int p=0; p<ans.size(); ++p){
    if(ans[p]!=0) return p;
  }
}

//MC simulation function

// [[Rcpp::export]]

List forward_day_d(NumericVector yj, NumericVector pj, NumericVector for_hours, NumericVector lambda, NumericVector x, NumericVector state, NumericVector l, NumericVector i, NumericVector st, NumericVector m, NumericVector store, NumericVector consec, NumericVector consec_calves, NumericVector alive, NumericVector death, int nsto, NumericVector calves, NumericVector agein, NumericVector x_calves, NumericVector l_calves, NumericVector i_calves, NumericVector st_calves, NumericVector resDl, NumericVector resRl, NumericVector resD2l, int simY, int y, int dd, NumericVector prod, IntegerVector env, NumericVector dist_locs, NumericVector responsive, NumericVector responsive_calves, NumericVector exposed, NumericVector resp, NumericVector exposed_calves, NumericVector resp_calves, float severity, NumericVector start_dist)
{
  //Declare variables
  int T, locs, feed_g, int_x, d, Mmax, tc, rho, gamma, aM, N, maxA, t, dy, dyn, tb, ti, tw, ds, nstates, sizel, t_il_t, x_lac_t, indexl, int_low_x;
  double mc, mC, int_m, cost_kg, Chg_tot, me, E, wb, lb, ww, lw, lgd, meat_cal, MRc, LEd, u, raamr, beta, psi, A, m50, Er, diff_l, intl, int_per, min_perc, death_perc, min_xc, max_C, xc_t, C_t, xc_tp, C_tp, raamr_t, beta_t, psi_t;
  NumericVector gtime, ms, Wc, Wc_d(334), propg(334), Chg(334), leand(212), w(3), ns, seqtbs, ts, per_lean, seq_length, percseq, xs_wide(25), Ctmp, xctmp, x_lactmp;
  double calf_s, fet_s;
  
  //General settings (see backward iteration for details)
  aM=6;
  N=90;
  maxA=aM+N;
  ns=sequence(0,N,1);
  NumericVector tbs(ns.size()), tis(ns.size()), tws(ns.size()), daysize(ns.size()), t_il(ns.size());
  locs=36;
  feed_g=15-1;
  tb=1-1;
  ti=32-1;
  tw=212-1;
  ds=40-1;
  T=tw+(aM+N)*365;
  nstates=4;
  ts=sequence(tw+1,T,1);
  NumericVector lt(ts.size()), leant(ts.size());
  for(int i = 0; i < ns.size(); ++i) {
    tbs[i]=tb+(aM +ns[i])*365;
    tis[i]=ti+(aM +ns[i])*365;
    tws[i]=tw+(aM +ns[i])*365;
    }
  seqtbs=sequence_inv(T-1,tbs[tbs.size()-1],1);
  for(int i = 0; i < ns.size(); ++i) {
    t_il[i]=(tws[i]-1)-seqtbs.size()+1;
  }
  float survy= 0.975;
  float surv=pow(survy,(1.0/365.0));
  calf_s=pow(0.875,1.0/212.0);
  fet_s=pow(1-0.135,1.0/334.0);
  mc=0.05;
  mC=0.44;
  int_m=0.03;
  ms=sequence(mc,mC+int_m,int_m);
  d = 100;
  gtime=sequence(1,334,1);
  Wc=pow(0.52*(gtime-73),3.0)/1000.0;
  Wc[Wc<0.001]=0;
  cost_kg=2940*4.184;
  for(int i = 0; i < (Wc.size()-1); ++i) {
    Wc_d[i]=(Wc[i+1]-Wc[i]);
    propg[i]=(Wc[i+1]-Wc[i])/max(Wc);
  }
  propg[Wc.size()-1]=NAN;
  Chg_tot=4400*4.184*pow(max(Wc),1.2);
  Chg=propg*Chg_tot;
  Mmax=220;
  me=4137*4.184;
  E=0.9;
  wb=Wc[Wc.size()-1];
  lb=wb*0.94;
  ww=17000+wb;
  lw=ww*0.73;
  lgd=(roundf(lw)-roundf(lb))/211;
  leand[0]=roundf(lb);
  for(int i = 1; i < 212; ++i) {
    leand[i]= leand[i-1] + lgd;
  }
  meat_cal=1500*4.184;
  LEd=lgd*meat_cal;
  MRc=200000*4.184;
  Er=MRc + LEd;
  u=31798;
  w[0]=10;
  w[1]=20;
  w[2]=30;
  tc=4; 
  rho=4184;
  A=0.84;
  for(int i = 0; i < ts.size(); ++i) {
    lt[i]=22*(1-exp(-0.240*((ts[i]/365)+4.5)));
    leant[i]=0.73*4.6*pow(lt[i],3.05);
  }
  diff_l=(leant[2]-leand[leand.size()-1])/3.0;
  leant[0]=leand[leand.size()-1]+diff_l*(1);
  leant[1]=leand[leand.size()-1]+diff_l*(2);
  leant[2]=leand[leand.size()-1]+diff_l*(3);
  sizel=leand.size()+leant.size();
  NumericVector lean_complete(sizel);
  for(int i = 0; i < lean_complete.size(); ++i) {
    if(i < (leand.size())){
      lean_complete[i]=leand[i];
    }else{
      indexl=i-leand.size();
      lean_complete[i]=leant[indexl];
    }
  }
  NumericVector leant_day(lean_complete.size()-1), lt_prewean(leand.size()), lt_complete(lean_complete.size()), max_perc(lean_complete.size()), C(lean_complete.size()), xc(lean_complete.size()), x_lac(lean_complete.size());
  for(int i = 0; i < (lean_complete.size()-1); ++i) {
    leant_day[i]=roundf((lean_complete[i+1]-lean_complete[i]));
  }
  intl=(0.94-0.73)/(leand.size()-1);
  per_lean=sequence_inv(0.94,0.73,intl);
  for(int i = 0; i < leand.size(); ++i) {
    lt_prewean[i]=pow(leand[i]/per_lean[i]/4.6,1/3.05);
  }
  for(int i = 0; i < lean_complete.size(); ++i) {
    if(i < leand.size()){
      lt_complete[i]=lt_prewean[i];
    }else{
      indexl=i-leand.size();
      lt_complete[i]=lt[indexl];
    }
  }
  seq_length=sequence(tw+1,tbs[0],1);
  int_per=(0.44-0.35)/(seq_length.size()-1);
  percseq=sequence_inv(0.44,0.35,int_per);
  for(int i = 0; i < lean_complete.size(); ++i) {
    if(i < tw){
      max_perc[i]=0.44;
    }else{
      if (i < tbs[0]){
        max_perc[i]=percseq[i-tw];
      }else{
        max_perc[i]=0.35;
      }
    }
  }
  min_perc=0.16;
  death_perc=0.05;
  int_x=2000;
  int_low_x=500;
  min_xc=roundToInteger(death_perc*lean_complete[0]/(1-death_perc),int_low_x);
  max_C=roundToInteger(max_perc[lean_complete.size()-1]*lean_complete[lean_complete.size()-1]/(1-max_perc[lean_complete.size()-1]),int_x);
  xs_wide[sequence(0,8,1)]=sequence(0,4000,int_low_x);
  xs_wide[9]=4700;
  xs_wide[10]=5300;
  xs_wide[sequence(11,13,1)]=sequence(6000,8000,1000);
  xs_wide[14]=8700;
  xs_wide[15]=9300;
  xs_wide[sequence(16,18,1)]=sequence(10000,12000,1000);
  xs_wide[sequence(19,24,1)]=sequence(14000,24000,2000);
  Ctmp=max_perc*lean_complete/(1-max_perc);
  x_lactmp=min_perc*lean_complete/(1-min_perc);
  xctmp=death_perc*lean_complete/(1-death_perc);
  for(int i = 0; i < lean_complete.size(); ++i) {
    C[i]=xs_wide[Closest(xs_wide,Ctmp[i])];
    xc[i]=xs_wide[Closest(xs_wide,xctmp[i])];
    x_lac[i]=xs_wide[Closest(xs_wide,x_lactmp[i])];
  }
  gamma=8.0;
  m50=0.27;
  int degrade=21;
  int m_tok=9;
  
  //Forward simulation
  t=dd+y*365-1;                     //subtract 1 for C++
  int age=t+365;                    //this is the age of the individual (they all start at age 366, -1 because C++, i.e. when t=0, age=365)
  int simT=365*simY;

  if(dd<32){
    dy=334+dd-1; //day of the calendar year; used as an index, so subtract 1 for C++
  }else{
    dy=dd-31-1;  //day of the calendar year; used as an index, so subtract 1 for C++
  }
  
  if(dy<(365-1)){ 
    dyn=dy+1;    //following day
  }else{
    dyn=0;  
  }

  //Calf loop
  IntegerVector calva=which_equal(calves,1);   //calves that are currently alive (for each female, post weaning)
  for (int cd=0; cd<calva.size(); ++cd){       //cd is the individual (loop over calves alive in that time step)
    int k=calva[cd]-1;                         //k is the id of the calf's mother
    int agec=dd-1;                             //calf age
    if(agec>tw){                               //start modelling calf separately only after it has reached the threshold for autonomous feeding (tw)
      //time dependent parameters
      xc_t=xc[agec];         //min blubber mass in this time step
      C_t=C[agec];           //max blubber mass in this time step
      xc_tp=xc[agec-1];      //min blubber mass in previous time step
      C_tp=C[agec-1];        //max blubber mass in previous time step 
      //calculate length-dependent parameters for current time step (metabolic costs are based on new size, after growth)
      raamr_t=26.23*pow(lt_complete[agec],-0.809); //in W/kg
      beta_t=0.0011*pow(lt_complete[agec],3.56);   //volume of water ingested in a lunge
      psi_t=0.47*pow(lt_complete[agec],2.88);      //forestomach capacity
      
      if(dd<365){                            //if it isn't the last day of the year (when final assessment of survival to year 1)
        //check if individual remains alive
        if(x_calves[k*365+agec]<=xc_tp){     //if starved
          calves[k]=0;                       //calf dies of starvation
          store[nsto*9+0]=k;                 //mother's ID
          store[nsto*9+1]=y;                 //year when born and weaned
          store[nsto*9+2]=agein[k];          //age when entering population  
          store[nsto*9+3]=R::rbinom(1,0.5);  //sex
          store[nsto*9+4]=1;                 //recruited
          store[nsto*9+5]=0;                 //didn't survive to 1 y
          store[nsto*9+6]=2;                 //cause of death for calf=2 (starvation after recruitment)
          store[nsto*9+7]=exposed_calves[k]; //# exposures post weaning
          store[nsto*9+8]=resp_calves[k];    //# disturbance events post weaning
          nsto=nsto+1;                       //increase counter for calves by 1
        }else{                                 //if calf is alive
          calves[k]=R::rbinom(1,surv);         //it can still die due to causes other than starvation and max age
          if(calves[k]==0){                    //store calf info, if it dies
            store[nsto*9+0]=k;                 //mother's ID
            store[nsto*9+1]=y;                 //year when born and weaned
            store[nsto*9+2]=agein[k];          //age when entering population  
            store[nsto*9+3]=R::rbinom(1,0.5);  //sex
            store[nsto*9+4]=1;                 //recruited
            store[nsto*9+5]=0;                 //didn't survive to 1 y
            store[nsto*9+6]=3;                 //cause of death for calf=3 (other)
            store[nsto*9+7]=exposed_calves[k]; //# exposure post weaning
            store[nsto*9+8]=resp_calves[k];    //# disturbance events post weaning
            nsto=nsto+1;                       //increase counter for calves by 1
          }else{                                                //if alive at the beginning and staying alive
            //extract indices for x
            int xi=ClosestLower(xs_wide, x_calves[k*365+agec]);
            IntegerVector xict=which_equal(xs_wide,xc_tp)-1;    //-1 because which_equal returns position using R indexing
            if(xi==xict[0]){
              xi=xi+1;                         //when above the critical mass, the calf doesn't immediately die
            }
            double xl=xs_wide[xi];
            int mi=m_tok;
            //select state transition and behavioural strategy
            int oldst=0; //previous state: juvenile
            st_calves[k*365+agec]=resDl[env[k]*xs_wide.size()+2*xs_wide.size()*mi+2*xs_wide.size()*ms.size()*l_calves[k*365+agec]+2*xs_wide.size()*ms.size()*(locs)*i_calves[k*365+agec]+2*xs_wide.size()*ms.size()*(locs)*3*oldst+2*xs_wide.size()*ms.size()*(locs)*3*nstates*0+xi]; //the last 0 corresponds to the year
            
            //pick random strategy when same fitness (see backward iteration functions for reference of what values>3 correspond to)
            if((st_calves[k*365+agec]>3)&&(l_calves[k*365+agec]!=1)){                      
              if(st_calves[k*365+agec]==4){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(1,3,1),1,TRUE); st_calves[k*365+agec]=sts[0];}  
              if(st_calves[k*365+agec]==5){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(1,2,1),1,TRUE); st_calves[k*365+agec]=sts[0]; }     
              if(st_calves[k*365+agec]==6){Rcpp::NumericVector ists(2); ists[0]=1; ists[1]=3; Rcpp::NumericVector sts=RcppArmadillo::sample(ists,1,TRUE); st_calves[k*365+agec]=sts[0];}   
              if(st_calves[k*365+agec]==7){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(2,3,1),1,TRUE); st_calves[k*365+agec]=sts[0];}     
            }
            if((st_calves[k*365+agec]>4)&&(l_calves[k*365+agec]==1)){                      
              if(st_calves[k*365+agec]==5){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(1,4,1),1,TRUE); st_calves[k*365+agec]=sts[0];} 
              if(st_calves[k*365+agec]==6){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(1,3,1),1,TRUE); st_calves[k*365+agec]=sts[0];}   
              if(st_calves[k*365+agec]==7){Rcpp::NumericVector ists(3); ists[0]=1; ists[1]=2; ists[2]=4; Rcpp::NumericVector sts=RcppArmadillo::sample(ists,1,TRUE); st_calves[k*365+agec]=sts[0];} 
              if(st_calves[k*365+agec]==8){Rcpp::NumericVector ists(3); ists[0]=1; ists[1]=3; ists[2]=4; Rcpp::NumericVector sts=RcppArmadillo::sample(ists,1,TRUE); st_calves[k*365+agec]=sts[0];}   
              if(st_calves[k*365+agec]==9){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(2,4,1),1,TRUE); st_calves[k*365+agec]=sts[0];}  
              if(st_calves[k*365+agec]==10){Rcpp::NumericVector ists(2); ists[0]=1; ists[1]=3; Rcpp::NumericVector sts=RcppArmadillo::sample(ists,1,TRUE); st_calves[k*365+agec]=sts[0];}   
              if(st_calves[k*365+agec]==11){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(1,2,1),1,TRUE); st_calves[k*365+agec]=sts[0];}   
              if(st_calves[k*365+agec]==12){Rcpp::NumericVector ists(2); ists[0]=1; ists[1]=4; Rcpp::NumericVector sts=RcppArmadillo::sample(ists,1,TRUE); st_calves[k*365+agec]=sts[0];}    
              if(st_calves[k*365+agec]==13){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(2,3,1),1,TRUE); st_calves[k*365+agec]=sts[0];}    
              if(st_calves[k*365+agec]==14){Rcpp::NumericVector ists(2); ists[0]=2; ists[1]=4; Rcpp::NumericVector sts=RcppArmadillo::sample(ists,1,TRUE); st_calves[k*365+agec]=sts[0];}   
              if(st_calves[k*365+agec]==15){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(3,4,1),1,TRUE); st_calves[k*365+agec]=sts[0];}   
            }
            
            //Disturbance
            double corrdist=t+start_dist[k];               //adjust time of disturbance depending on where along the timeline that individual is located (for random disturbance events)
            if(corrdist>simT){corrdist=corrdist-(simT+1);} //wrap timeline around when beyond max time
            double prd=dist_locs[l_calves[k*365+agec]*simT+corrdist]; //extract proportion of that location that is disturbed on that day (adjusting by timing for that individual)
            int respc;
            int expo=R::rbinom(1,prd);                    //use proportion of location disturbed to determine whether that individual is exposed or not on that day
            if((st_calves[k*365+agec]==1) && (expo==1)){  //if optimal behaviour is to stay but location is disturbed that day and individual is exposed
              if(l_calves[k*365+agec]>1){                 
                exposed_calves[k]=exposed_calves[k]+1;    //counter of exposures increases                              
                double prr=responsive_calves[k];
                int res=R::rbinom(1,prr);                 //response depends on responsiveness (if responsive, she leaves)
                if(res==1){                               //if response, select second best strategy
                  respc=resD2l[env[k]*xs_wide.size()+2*xs_wide.size()*mi+2*xs_wide.size()*ms.size()*l_calves[k*365+agec]+2*xs_wide.size()*ms.size()*(locs)*i_calves[k*365+agec]+2*xs_wide.size()*ms.size()*(locs)*3*oldst+2*xs_wide.size()*ms.size()*(locs)*3*nstates*0+xi];                       //if disturbed from loc 3 onwards, they can respond by picking second to best strategy
                  resp_calves[k]=resp_calves[k]+1;
                  if((respc==1) || (respc>3)){            //if second best choice is again to stay (e.g. because of random pick above), pick another strategy randomly
                    Rcpp::NumericVector rss=RcppArmadillo::sample(sequence(2,3,1),1,TRUE);
                    respc=rss[0];
                  }
                }else{                                    //otherwise, she stays
                  respc=1;     
                }
                st_calves[k*365+agec]=respc;              //behaviour is adjusted in response to disturbance
              }
            }
            if((l_calves[k*365+agec]>=(locs-1)) && (st_calves[k*365+agec]==2)){
              st_calves[k*365+agec]=1;                    //if in the last possible location, she will not move
            }
            
            //Calf behavioural choices
            //strategy 1
            if(st_calves[k*365+agec]==1){                          //if strategy is to stay and feed/rest
              consec_calves[k]=consec_calves[k]+1;                 //increase number of consecutive days in that patch by 1
              if (consec_calves[k]<= degrade){                     //if she has been in that patch for less than 3 weeks
                i_calves[k*365+agec+1]=i_calves[k*365+agec];       //patch type doesn't change
              }else{
                consec_calves[k]=0;                                //reset counter for consecutive days in patch (since patch changes)
                i_calves[k*365+agec+1]=2;                          //and patch type gets degraded
              }
              l_calves[k*365+agec+1]=l_calves[k*365+agec];
              Rcpp::NumericVector yju(30);
              Rcpp::NumericVector pju(30);
              for (int yjj=0; yjj<30; ++yjj){
                yju[yjj]=yj[env[k]*3 + 2*3*yjj + 2*3*30*l_calves[k*365+agec] + 2*3*30*(locs+1)*dy + i_calves[k*365+agec]];
                pju[yjj]=pj[env[k]*3 + 2*3*yjj + 2*3*30*l_calves[k*365+agec] + 2*3*30*(locs+1)*dy + i_calves[k*365+agec]];
              }
              float prey=yju[oneMultinom(pju)]*(prod[k]);         //pick a random amount of food found in patch
              if(respc==1){                                       //if the location is disturbed, the whale is exposed but she doesn't move away (i.e. she stays in the disturbed location)
                prey=prey*severity;                               //the productivity of the cell is decreased, based on severity
              }
              float for_hoursu;
              for_hoursu=for_hours[l_calves[k*365+agec]*365+dy];  //number of hours available to feed in that location given twilight times
              Rcpp::NumericVector preys(1);
              preys[0]=prey;
              Rcpp::NumericVector cfs=cf(x_calves[k*365+agec]-grow(leant_day[agec], meat_cal, u), lean_complete[agec], raamr_t, preys, w, beta_t, psi_t, tc, l_calves[k*365+agec], agec, for_hoursu);
              Rcpp::NumericVector crs=cr(x_calves[k*365+agec]-grow(leant_day[agec], meat_cal, u), lean_complete[agec], preys, w, beta_t, psi_t, tc, l_calves[k*365+agec], agec, for_hoursu);
              Rcpp::NumericVector foods=food(preys, w, beta_t, psi_t, tc, l_calves[k*365+agec], agec, for_hoursu, rho, A);
              x_calves[k*365+agec+1]=x_calves[k*365+agec]-grow(leant_day[agec], meat_cal, u)+(- cfs[0] - crs[0] + foods[0])/u;  //update blubber accordingly
              if(x_calves[k*365+agec+1]>C_t){x_calves[k*365+agec+1]=C_t;} //if blubber greater than max, set to max in this time step 
            } //close strategy 1
            
            //strategy 2
            if(st_calves[k*365+agec]==2){                         //if strategy is to move forward
              consec_calves[k]=0;                                 //reset number of consecutive days in patch
              l_calves[k*365+agec+1]=l_calves[k*365+agec]+1;      //update location
              Rcpp::NumericVector lambda_f(3);
              for (int la=0; la<3; ++la){
                lambda_f[la]=lambda[env[k]*(locs+1) + 2*(locs+1)*la + 2*(locs+1)*3*dyn + l_calves[k*365+agec+1]];  //lambda depends on location and time
              }
              i_calves[k*365+agec+1]=oneMultinom(lambda_f);       //pick a patch type reached in the new location
              x_calves[k*365+agec+1]=x_calves[k*365+agec]-grow(leant_day[agec], meat_cal, u)-cot(x_calves[k*365+agec]-grow(leant_day[agec], meat_cal, u),lean_complete[agec])*d*(x_calves[k*365+agec]-grow(leant_day[agec], meat_cal, u)+lean_complete[agec])/u;         //update blubber accordingly
              if(x_calves[k*365+agec+1]>C_t){x_calves[k*365+agec+1]=C_t;} //if blubber greater than max, set to max in this time step 
            } //close strategy 2
            
            //strategy 3/4
            if((st_calves[k*365+agec]==3)||((l_calves[k*365+agec]==1)&&(st_calves[k*365+agec]==4))){  //if strategy is to move backward or within breeding ground
              consec_calves[k]=0;                                //reset number of consecutive days in patch
              if((l_calves[k*365+agec]==0)||((l_calves[k*365+agec]==1)&&(st_calves[k*365+agec]==4))){ //if in breeding ground
                l_calves[k*365+agec+1]=l_calves[k*365+agec];                                          //she moves within
              }else{
                l_calves[k*365+agec+1]=l_calves[k*365+agec]-1;                                        //otherwise location decreases by 1
              }
              Rcpp::NumericVector lambda_f(3);
              for (int la=0; la<3; ++la){
                lambda_f[la]=lambda[env[k]*(locs+1) + 2*(locs+1)*la + 2*(locs+1)*3*dyn + l_calves[k*365+agec+1]];  //lambda depends on location and time
              }
              i_calves[k*365+agec+1]=oneMultinom(lambda_f);                                          //pick a patch type reached in the new location
              x_calves[k*365+agec+1]=x_calves[k*365+agec]-grow(leant_day[agec], meat_cal, u)-cot(x_calves[k*365+agec]-grow(leant_day[agec], meat_cal, u),lean_complete[agec])*d*(x_calves[k*365+agec]-grow(leant_day[agec], meat_cal, u)+lean_complete[agec])/u;  //update blubber accordingly
              if(x_calves[k*365+agec+1]>C_t){x_calves[k*365+agec+1]=C_t;}                            //if blubber greater than max, set to max in this time step
            } //close strategy 3/4

          } 
        } 
      }else{ //if it is the last day of the year
        if(x_calves[k*365+agec]<=xc_tp){     //if starved
          calves[k]=0;                       //calf dies of starvation
          store[nsto*9+0]=k;                 //mother's ID
          store[nsto*9+1]=y;                 //year when born and weaned
          store[nsto*9+2]=agein[k];          //age when entering population  
          store[nsto*9+3]=R::rbinom(1,0.5);  //sex
          store[nsto*9+4]=1;                 //recruited
          store[nsto*9+5]=0;                 //didn't survive to 1 y
          store[nsto*9+6]=2;                 //cause of death for calf=2 (starvation after recruitment)
          store[nsto*9+7]=exposed_calves[k]; //# exposures post weaning
          store[nsto*9+8]=resp_calves[k];    //# disturbance events post weaning
          nsto=nsto+1;                       //increase counter for calves by 1
        }else{                               //if calf is alive
          store[nsto*9+0]=k;                 //mother's ID
          store[nsto*9+1]=y;                 //year when born and weaned
          store[nsto*9+2]=agein[k];          //age when entering population  
          store[nsto*9+3]=R::rbinom(1,0.5);  //sex
          store[nsto*9+4]=1;                 //recruited
          calves[k]=R::rbinom(1,surv);       //survives last day?
          if (calves[k]==1){                 //if it survives
            store[nsto*9+5]=1;               //survived to 1 y
            store[nsto*9+6]=0;               //cause of death for calf=0 (didnt die)
          }else{
            store[nsto*9+5]=0;               //didnt survive to 1 y
            store[nsto*9+6]=3;               //cause of death for calf=3 (other)
          }
          store[nsto*9+7]=exposed_calves[k]; //exposure post weaning
          store[nsto*9+8]=resp_calves[k];    //disturbance post weaning
          nsto=nsto+1;                       //increase counter for calves by 1
        }
      } 
    } 
  } //close calf loop
  
  //Female loop  
  IntegerVector inda=which_equal(alive,1);   //alive individuals at the beginning of each day 
  for (int kd=0; kd<inda.size(); ++kd){      //k is the individual (loop over individuals alive in that time step)
      int k=inda[kd]-1;
      //time indices
      int a=floor(age/365);                      //age of the whale in years (this can be used as is as an index for arrays)
      int n=floor(((age+1)-(365*(aM-1)))/365)-1; //potential number of implantations until now
      if(age<tbs[0]){n=-1;}                      //for juveniles (it doesn't matter because they do not lactate)
      if(n>(-1)){
        if(age>=tbs[n]){
          t_il_t=t_il[n];     //index to extract current day of lactation (t-t_il_t)
        }else{
          t_il_t=t_il[n-1];   //if calf has been lactating after avg weaning date
        }
      }
      //time dependent parameters
      xc_t=xc[age];         //min blubber mass in this time step
      C_t=C[age];           //max blubber mass in this time step
      xc_tp=xc[age-1];      //min blubber mass in previous time step
      C_tp=C[age-1];        //max blubber mass in previous time step
      x_lac_t=x_lac[age];   //threshold mass for lactation in this time step
      
      //calculate length-dependent parameters for current time step (metabolic costs are based on new size, after growth)
      raamr_t=26.23*pow(lt_complete[age],-0.809); //in W/kg
      beta_t=0.0011*pow(lt_complete[age],3.56);   //volume of water ingested in a lunge
      psi_t=0.47*pow(lt_complete[age],2.88);      //forestomach capacity
      
      //check if individual remains alive
      if(x[k*simT+t]<=xc_tp){                     //if she starved
        alive[k]=0;                               //she dies
        death[k]=1;                               //cause of death: starvation
        if(state[k*simT+t]==3){                   //if she was lactating
            double fms, Erem, bm;
            int tri;
            if((age-t_il_t)<=tw){                 //if female death occurs before the calf can feed autonomously
              tri=tws[n]-age;                     //days to autonomous feeding
              Erem=Er*(tri)/u;                    //blubber required to survive and grow up to 7 months (without milk from mother)
              bm=leand[age-t_il_t]*m[k*simT+t]/(1-m[k*simT+t]);    //current blubber mass, derived from % of blubber and lean mass in previous time step
              fms=(bm-Erem)/(leand[leand.size()-1]+(bm-Erem));     //final % blubber mass, given current mass and mass required to survive and grow up to 7 months
              if(fms<mc){fms=mc;}                                  //if % below the minimum threshold, set to minimum
            }else{                                //if female death occurs after the threshold for autonomous feeding
              fms=m[k*simT+t];                    //the final blubber proportion corresponds to the current proportion, because calf can autonomously feed straight away
              tri=1;
            }
            double pri= pow(calf_s,tri)*(pow(fms,gamma)/(pow(fms,gamma) + pow(m50,gamma))); //probability of recruiting in the population given condition
            int recr=R::rbinom(1,pri);                         //recruits or not?
            if (recr==1){                                      //if it recruits
                calves[k]=1;                                   //it needs to be modelled until 1y old
                agein[k]=age-t_il_t+1;                         //age when entering population
                responsive_calves[k]=R::rbeta(5,5);            //draw a level of responsiveness for new individual
                consec_calves[k]=consec[k];                    //inherits consecutive number of days in current patch from mother
                //set relevant vectors for new individual
                if((age-t_il_t)<=tw){                          //if before it can feed autonomously
                  if((bm-Erem)>xc[tw]){
                    x_calves[k*365+tw+1]=bm-Erem;
                  }else{
                    x_calves[k*365+tw+1]=xc[tw];
                  } //blubber mass is mass at weaning
                  l_calves[k*365+tw+1]=l[k*simT+t];            //location is where calf lost the mother
                  i_calves[k*365+tw+1]=i[k*simT+t];            //patch type is the same
                }else{                                         //if it can already feed autonomously, it enters the population in the next time step
                  x_calves[k*365+age-t_il_t+1]=lean_complete[age-t_il_t]*m[k*simT+t]/(1-m[k*simT+t]);                  //blubber mass is current blubber mass
                  if(x_calves[k*365+age-t_il_t+1]>C[age-t_il_t+1]){x_calves[k*365+age-t_il_t+1]=C[age-t_il_t+1];}      //correct blubber mass if too high
                  if(x_calves[k*365+age-t_il_t+1]<xc[age-t_il_t+1]){x_calves[k*365+age-t_il_t+1]=xc[age-t_il_t+1];}    //correct blubber mass if too low
                  l_calves[k*365+age-t_il_t+1]=l[k*simT+t];                                            //location is where she lost the mother
                  i_calves[k*365+age-t_il_t+1]=i[k*simT+t];                                            //patch type is the same
                }
            }else{
              store[nsto*9+0]=k;                 //mother's ID
              store[nsto*9+1]=y;                 //year when born and weaned
              store[nsto*9+2]=age-t_il_t+1;      //age when entering population  
              store[nsto*9+3]=R::rbinom(1,0.5);  //sex
              store[nsto*9+4]=0;                 //didn't recruit
              store[nsto*9+5]=0;                 //didn't survive to 1 y
              store[nsto*9+6]=1;                 //cause of death for calf=1 (didn't recruit)
              store[nsto*9+7]=0;                 //# exposures post weaning
              store[nsto*9+8]=0;                 //# disturbance events post weaning
              nsto=nsto+1;                       //increase counter for calves by 1
            }
        }
      }else{                                     //if female is alive
        alive[k]=R::rbinom(1,surv);              //she can still die due to causes other than starvation and max age
        if(alive[k]==0){
          death[k]=3;                            //cause of death - other
          if(state[k*simT+t]==3){                //if she was lactating
              double fms, Erem, bm;
              int tri;
              if((age-t_il_t)<=tw){              //if female death occurs before the calf can feed autonomously
                tri=tws[n]-age;
                Erem=Er*(tri)/u;                 //blubber required to survive and grow up to 7 months (without milk from mother)
                bm=leand[age-t_il_t]*m[k*simT+t]/(1-m[k*simT+t]);   //current blubber mass, derived from % of blubber and lean mass in previous time step
                fms=(bm-Erem)/(leand[leand.size()-1]+(bm-Erem));    //final % blubber mass, given current mass and mass required to survive and grow up to 7 months
                if(fms<mc){fms=mc;}                                 //if condition below the minimum threshold, set to minimum
              }else{                            //if female death occurs after the threshold for autonomous feeding
                fms=m[k*simT+t];                //the final blubber proportion corresponds to the current proportion because calf can autonomously feed straight away
                tri=1;
              }
              double pri= pow(calf_s,tri)*(pow(fms,gamma)/(pow(fms,gamma) + pow(m50,gamma)));
              int recr=R::rbinom(1,pri);                       //recruits or not?
              if (recr==1){                                    //if it recruits
                calves[k]=1;                                   //it needs to be modelled until day 365
                agein[k]=age-t_il_t+1;                         //age when entering population
                responsive_calves[k]=R::rbeta(5,5);            //draw a level of responsiveness for new individual
                consec_calves[k]=consec[k];                    //inherits consecutive number of days in current patch from mother
                //set relevant vectors for new individual
                if((age-t_il_t)<=tw){                          //if before it can feed autonomously
                  if((bm-Erem)>xc[tw]){
                    x_calves[k*365+tw+1]=bm-Erem;
                  }else{
                    x_calves[k*365+tw+1]=xc[tw];
                  } //blubber mass is mass at weaning
                  l_calves[k*365+tw+1]=l[k*simT+t];            //location is where she lost the mother
                  i_calves[k*365+tw+1]=i[k*simT+t];            //patch type is the same
                }else{                                         //if it can already feed autonomously, it enters the population in the next time step
                  x_calves[k*365+age-t_il_t+1]=lean_complete[age-t_il_t]*m[k*simT+t]/(1-m[k*simT+t]);                  //blubber mass is current blubber mass
                  if(x_calves[k*365+age-t_il_t+1]>C[age-t_il_t+1]){x_calves[k*365+age-t_il_t+1]=C[age-t_il_t+1];}      //correct blubber mass if too high
                  if(x_calves[k*365+age-t_il_t+1]<xc[age-t_il_t+1]){x_calves[k*365+age-t_il_t+1]=xc[age-t_il_t+1];}    //correct blubber mass if too low
                  l_calves[k*365+age-t_il_t+1]=l[k*simT+t];    //location is where she lost the mother
                  i_calves[k*365+age-t_il_t+1]=i[k*simT+t];    //patch type is the same
                }
              }else{
                store[nsto*9+0]=k;                 //mother's ID
                store[nsto*9+1]=y;                 //year when born and weaned
                store[nsto*9+2]=age-t_il_t+1;      //age when entering population  
                store[nsto*9+3]=R::rbinom(1,0.5);  //sex
                store[nsto*9+4]=0;                 //didn't recruit
                store[nsto*9+5]=0;                 //didn't survive to 1 y
                store[nsto*9+6]=1;                 //cause of death for calf=1 (didn't recruit)
                store[nsto*9+7]=0;                 //# exposures post weaning
                store[nsto*9+8]=0;                 //# disturbance events post weaning
                nsto=nsto+1;                       //increase counter for calves by 1
              }
          }
        }else{                                     //if alive at the beginning and staying alive
          //extract indices for x and m
          int xi=ClosestLower(xs_wide, x[k*simT+t]);
          IntegerVector xict=which_equal(xs_wide,xc_tp)-1;  //-1 because which_equal returns a value based on R indexing
          if(xi==xict[0]){
            xi=xi+1;       //when above the critical mass, the female doesn't immediately die - use the following mass bin
          }
          double xl=xs_wide[xi];
          int mi;
          if (state[k*simT+t]==3){ //if lactating
            mi=(m[k*simT+t]-mc)/int_m;
            float ml=ms[mi];
            if(ml==mc){
              ml=mc+int_m; //when above the critical % mass, the calf doesn't immediately die - use the following mass bin
              mi=mi+1;
            }
          }else{
            mi=m_tok;
          }
          //select state transition and behavioural strategy
          int oldst=state[k*simT+t];                //temporarily save initial state
          st[k*simT+t]=resDl[env[k]*xs_wide.size()+2*xs_wide.size()*mi+2*xs_wide.size()*ms.size()*l[k*simT+t]+2*xs_wide.size()*ms.size()*(locs)*i[k*simT+t]+2*xs_wide.size()*ms.size()*(locs)*3*oldst+2*xs_wide.size()*ms.size()*(locs)*3*nstates*a+xi]; 
          state[k*simT+t]=resRl[env[k]*xs_wide.size()+2*xs_wide.size()*mi+2*xs_wide.size()*ms.size()*l[k*simT+t]+2*xs_wide.size()*ms.size()*(locs)*i[k*simT+t]+2*xs_wide.size()*ms.size()*(locs)*3*oldst+2*xs_wide.size()*ms.size()*(locs)*3*nstates*a+xi]-1; //-1 because used as index in Rcpp;
          int oldst2=state[k*simT+t];               //temporarily save state pre random selection 
          
          //pick random state when same fitness 
          if(state[k*simT+t]==4){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(1,2,1),1,TRUE); state[k*simT+t]=sts[0];}
          if(state[k*simT+t]==5){Rcpp::NumericVector ists(2); ists[0]=1; ists[1]=3; Rcpp::NumericVector sts=RcppArmadillo::sample(ists,1,TRUE); state[k*simT+t]=sts[0];}

          //fetus and calf mortality from causes other than the mother's energetic state
          if(state[k*simT+t]==2){  //if pregnant, possibility of abortion of fetus
            double fsurv=R::rbinom(1,fet_s);
            if(fsurv==0){
              state[k*simT+t]=1;
            }
          }
          double csurv=1;          //set it to 1 by default 
          if(state[k*simT+t]==3){  //if lactating, possibility of death of calf
            csurv=R::rbinom(1,calf_s);
            if(csurv==0){
              state[k*simT+t]=1;
            }
          }
          if((st[k*simT+t]==4)&&(oldst2==4)&&(state[k*simT+t]==1)){ //pick strategy again in case she is pregnant on last day, and birth vs. abortion have same fitness, and chance has picked abortion (she needs a behavioural strategy in that case)
            st[k*simT+t]=resDl[env[k]*xs_wide.size()+2*xs_wide.size()*mi+2*xs_wide.size()*ms.size()*l[k*simT+t]+2*xs_wide.size()*ms.size()*(locs)*i[k*simT+t]+2*xs_wide.size()*ms.size()*(locs)*3*state[k*simT+t]+2*xs_wide.size()*ms.size()*(locs)*3*nstates*a+xi]; 
            }
          //pick random strategy when same fitness
          if((st[k*simT+t]>3)&&(l[k*simT+t]!=1)){                      
            if(st[k*simT+t]==4){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(1,3,1),1,TRUE); st[k*simT+t]=sts[0];}   
            if(st[k*simT+t]==5){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(1,2,1),1,TRUE); st[k*simT+t]=sts[0]; }     
            if(st[k*simT+t]==6){Rcpp::NumericVector ists(2); ists[0]=1; ists[1]=3; Rcpp::NumericVector sts=RcppArmadillo::sample(ists,1,TRUE); st[k*simT+t]=sts[0];}    
            if(st[k*simT+t]==7){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(2,3,1),1,TRUE); st[k*simT+t]=sts[0];}     
          }
          if((st[k*simT+t]>4)&&(l[k*simT+t]==1)){                    
            if(st[k*simT+t]==5){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(1,4,1),1,TRUE); st[k*simT+t]=sts[0];} 
            if(st[k*simT+t]==6){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(1,3,1),1,TRUE); st[k*simT+t]=sts[0];}  
            if(st[k*simT+t]==7){Rcpp::NumericVector ists(3); ists[0]=1; ists[1]=2; ists[2]=4; Rcpp::NumericVector sts=RcppArmadillo::sample(ists,1,TRUE); st[k*simT+t]=sts[0];}   
            if(st[k*simT+t]==8){Rcpp::NumericVector ists(3); ists[0]=1; ists[1]=3; ists[2]=4; Rcpp::NumericVector sts=RcppArmadillo::sample(ists,1,TRUE); st[k*simT+t]=sts[0];}   
            if(st[k*simT+t]==9){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(2,4,1),1,TRUE); st[k*simT+t]=sts[0];}  
            if(st[k*simT+t]==10){Rcpp::NumericVector ists(2); ists[0]=1; ists[1]=3; Rcpp::NumericVector sts=RcppArmadillo::sample(ists,1,TRUE); st[k*simT+t]=sts[0];}  
            if(st[k*simT+t]==11){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(1,2,1),1,TRUE); st[k*simT+t]=sts[0];}   
            if(st[k*simT+t]==12){Rcpp::NumericVector ists(2); ists[0]=1; ists[1]=4; Rcpp::NumericVector sts=RcppArmadillo::sample(ists,1,TRUE); st[k*simT+t]=sts[0];}   
            if(st[k*simT+t]==13){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(2,3,1),1,TRUE); st[k*simT+t]=sts[0];}   
            if(st[k*simT+t]==14){Rcpp::NumericVector ists(2); ists[0]=2; ists[1]=4; Rcpp::NumericVector sts=RcppArmadillo::sample(ists,1,TRUE); st[k*simT+t]=sts[0];}  
            if(st[k*simT+t]==15){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(3,4,1),1,TRUE); st[k*simT+t]=sts[0];}   
          }
          
          //Disturbance
          double corrdist=t+start_dist[k];                 //adjust time of disturbance depending on where along the timeline that individual is located (for random disturbance events)
          if(corrdist>simT){corrdist=corrdist-(simT+1);}   //wrap timeline around when beyond max
          double prd=dist_locs[l[k*simT+t]*simT+corrdist]; //extract proportion of that location that is disturbed on that day
          int expo=R::rbinom(1,prd);                       //use proportion to determine whether that individual is exposed or not on that day
          if((st[k*simT+t]==1) && (expo==1)){              //if optimal behaviour is to stay but location is disturbed that day
            if(l[k*simT+t]>1){                           
              exposed[k*simT+t]=1;                         //they are exposed on that day
              double prr=responsive[k];
              int res=R::rbinom(1,prr);
              if(res==1){                                  //response depends on responsiveness (if responsive, she leaves)
                resp[k*simT+t]=resD2l[env[k]*xs_wide.size()+2*xs_wide.size()*mi+2*xs_wide.size()*ms.size()*l[k*simT+t]+2*xs_wide.size()*ms.size()*(locs)*i[k*simT+t]+2*xs_wide.size()*ms.size()*(locs)*3*state[k*simT+t]+2*xs_wide.size()*ms.size()*(locs)*3*nstates*a+xi];  //if response, she picks second to best strategy
                if((resp[k*simT+t]==1) || (resp[k*simT+t]>3)){    //if second best choice is again to stay (because of random pick above), pick another strategy randomly
                  Rcpp::NumericVector rss=RcppArmadillo::sample(sequence(2,3,1),1,TRUE);
                  resp[k*simT+t]=rss[0];
                }
              }else{ //otherwise, she stays
                resp[k*simT+t]=1;     
              }
              st[k*simT+t]=resp[k*simT+t];   //behaviour is adjusted in response to disturbance
            }
          }
          
          if((l[k*simT+t]>=(locs-1)) && (st[k*simT+t]==2)){
            st[k*simT+t]=1;                  //if in the last possible location, she will not move
          }
          
          //calf abandonment and weaning
          if((oldst==3) && (state[k*simT+t]==1) && (csurv==1)){  //if she is still lactating, but best strategy is to wean the calf (and calf is alive)
            double fms, Erem, bm;
            int tri;
            if((age-t_il_t)<=tw){                                //if this occurs before the calf can feed autonomously
              tri=tws[n]-age;
              Erem=Er*(tri)/u;                                   //blubber required to survive and grow up to 7 months (without milk from mother)
              bm=leand[age-t_il_t]*m[k*simT+t]/(1-m[k*simT+t]);  //current blubber mass, derived from % of blubber and lean mass in previous time step
              fms=(bm-Erem)/(leand[leand.size()-1]+(bm-Erem));   //final % blubber mass, given current mass and mass required to survive and grow up to 7 months
              if(fms<mc){fms=mc;}                                //if % blubber below the minimum threshold, set to minimum
            }else{                                               //if this occurs after the threshold for autonomous feeding
              fms=m[k*simT+t];                                   //the final proportion corresponds to the current proportion because calf can autonomously feed straight away
              tri=1;
            }
            double pri= pow(calf_s,tri)*(pow(fms,gamma)/(pow(fms,gamma) + pow(m50,gamma)));
            int recr=R::rbinom(1,pri);                       //recruits or not?
            if (recr==1){                                    //if it recruits
              calves[k]=1;                                   //it must be modelled until day 365
              agein[k]=age-t_il_t+1;                         //age when entering population
              responsive_calves[k]=R::rbeta(5,5);            //draw a level of responsiveness for new individual
              consec_calves[k]=consec[k];                    //inherits consecutive number of days in current patch from mother
              //set relevant vectors for new individual 
              if((age-t_il_t)<=tw){                          //if before it can feed autonomously
                if((bm-Erem)>xc[tw]){
                  x_calves[k*365+tw+1]=bm-Erem;
                }else{
                  x_calves[k*365+tw+1]=xc[tw];
                } //blubber mass is mass at weaning
                l_calves[k*365+tw+1]=l[k*simT+t];            //location is where she lost the mother
                i_calves[k*365+tw+1]=i[k*simT+t];            //patch type is the same
              }else{                                         //if it can already feed autonomously, it enters the population in the next time step
                x_calves[k*365+age-t_il_t+1]=lean_complete[age-t_il_t]*m[k*simT+t]/(1-m[k*simT+t]);                  //blubber mass is current blubber mass
                if(x_calves[k*365+age-t_il_t+1]>C[age-t_il_t+1]){x_calves[k*365+age-t_il_t+1]=C[age-t_il_t+1];}      //correct blubber mass if too high
                if(x_calves[k*365+age-t_il_t+1]<xc[age-t_il_t+1]){x_calves[k*365+age-t_il_t+1]=xc[age-t_il_t+1];}    //correct blubber mass if too low
                l_calves[k*365+age-t_il_t+1]=l[k*simT+t];    //location is where she lost the mother
                i_calves[k*365+age-t_il_t+1]=i[k*simT+t];    //patch type is the same
              }
            }else{
              store[nsto*9+0]=k;                 //mother's ID
              store[nsto*9+1]=y;                 //year when born and weaned
              store[nsto*9+2]=age-t_il_t+1;      //age when entering population  
              store[nsto*9+3]=R::rbinom(1,0.5);  //sex
              store[nsto*9+4]=0;                 //didn't recruit
              store[nsto*9+5]=0;                 //didn't survive to 1 y
              store[nsto*9+6]=1;                 //cause of death for calf=1 (didn't recruit)
              store[nsto*9+7]=0;                 //# exposures post weaning
              store[nsto*9+8]=0;                 //# disturbance events post weaning
              nsto=nsto+1;                       //increase counter for calves by 1
            }
          }
          
          //calf recruitment at max weaning time
          if((oldst==3) && contains(tbs-1,age)){  //if she is still lactating on day before next birth, the calf is now weaned
            store[nsto*9+0]=k;                    //mother's ID
            store[nsto*9+1]=y;                    //year when born and weaned
            store[nsto*9+2]=366;                  //age when entering population  
            store[nsto*9+3]=R::rbinom(1,0.5);     //sex
            double pri=calf_s*(pow(m[k*simT+t],gamma)/(pow(m[k*simT+t],gamma) + pow(m50,gamma))); //probability of recruitment based on condition at weaning
            int recr=R::rbinom(1,pri);            //recruits or not?
            store[nsto*9+4]=recr;                 
            if (recr==1){                         //if it recruits
              store[nsto*9+5]=1;                  //survived to 1 y
              store[nsto*9+6]=0;                  //cause of death for calf=0 (didn't die)
            }else{
              store[nsto*9+5]=0;                  //didn't survive to 1 y
              store[nsto*9+6]=1;                  //cause of death for calf=1 (didn't recruit)
            }
            store[nsto*9+7]=0;                    //# exposures post weaning
            store[nsto*9+8]=0;                    //# disturbance events post weaning
            nsto=nsto+1;                          //increase counter for calves by 1
          }
          
          //if day before birth
          if((oldst==2) && (state[k*simT+t]==3)){
            st[k*simT+t]=0;
            i[k*simT+t+1]=i[k*simT+t];
            l[k*simT+t+1]=l[k*simT+t];
            double prey=0;                         //female doesn't feed on day she gives birth
            float for_hoursu;
            for_hoursu=for_hours[l[k*simT+t]*365+dy];
            Rcpp::NumericVector preys(1);
            preys[0]=prey;
            Rcpp::NumericVector crs=cr(x[k*simT+t]-grow(leant_day[age], meat_cal, u)+ Wc[Wc.size()-1], lean_complete[age], preys, w, beta_t, psi_t, tc, l[k*simT+t], age, for_hoursu);
            x[k*simT+t+1]=x[k*simT+t]-grow(leant_day[age], meat_cal, u)+(-crs[0] - ag(Wc.size()-2,Wc_d, cost_kg, propg, Chg_tot))/u;
            if(x[k*simT+t+1]>C_t){x[k*simT+t+1]=C_t;}  //if blubber greater than max, set to max in this time step 
            state[k*simT+t+1]=3;
            m[k*simT+t+1]=mc+int_m;
          } //close day before birth if
          
          //strategy 1
          if(st[k*simT+t]==1){                          //if strategy is to stay and feed/rest
            consec[k]=consec[k]+1;                      //increase number of consecutive days in that patch by 1
            if (consec[k]<= degrade){                   //if she has been in that patch for less than 3 weeks
              i[k*simT+t+1]=i[k*simT+t];                //patch type doesn't change
            }else{
              consec[k]=0;                              //reset counter for consecutive days in patch (since patch changes)
              i[k*simT+t+1]=2;                          //and patch type gets degraded
            }
            l[k*simT+t+1]=l[k*simT+t];
            Rcpp::NumericVector yju(30);
            Rcpp::NumericVector pju(30);
            for (int yjj=0; yjj<30; ++yjj){
              yju[yjj]=yj[env[k]*3 + 2*3*yjj + 2*3*30*l[k*simT+t] + 2*3*30*(locs+1)*dy + i[k*simT+t]];
              pju[yjj]=pj[env[k]*3 + 2*3*yjj + 2*3*30*l[k*simT+t] + 2*3*30*(locs+1)*dy + i[k*simT+t]];
            }
            float prey=yju[oneMultinom(pju)]*(prod[k]); //pick a random amount of food found on patch
            if(resp[k*simT+t]==1){ //if the location is disturbed, the whale is exposed but she doesn't move away (i.e. she stays in the disturbed location):
              prey=prey*severity;  //the productivity of that location is decreased based on severity
            }
            float for_hoursu;
            for_hoursu=for_hours[l[k*simT+t]*365+dy];
            Rcpp::NumericVector preys(1);
            preys[0]=prey;
            
            if (state[k*simT+t]==3){ //if she is lactating
              Rcpp::NumericVector cfs=cf(x[k*simT+t]-grow(leant_day[age], meat_cal, u), lean_complete[age], raamr_t, preys, w, beta_t, psi_t, tc, l[k*simT+t], age, for_hoursu);
              Rcpp::NumericVector crs=cr(x[k*simT+t]-grow(leant_day[age], meat_cal, u), lean_complete[age], preys, w, beta_t, psi_t, tc, l[k*simT+t], age, for_hoursu);
              Rcpp::NumericVector foods=food(preys, w, beta_t, psi_t, tc, l[k*simT+t], age, for_hoursu, rho, A);
              double test1=x[k*simT+t]-grow(leant_day[age], meat_cal, u) + (-cfs[0] - crs[0] + foods[0])/u - x_lac_t;
              double test2=(x[k*simT+t]-grow(leant_day[age], meat_cal, u)-x_lac_t)*u - cfs[0] - crs[0] + foods[0] - Mmax*me/E;
              if(test1<=0){
                x[k*simT+t+1] = x[k*simT+t]-grow(leant_day[age], meat_cal, u)+(- cfs[0] - crs[0] + foods[0])/u;
                float milk = 0;
                m[k*simT+t+1] = blub_perc(milk,age,m[k*simT+t],me,Er,u,t_il_t,lean_complete,leant_day,tw,meat_cal);
                if(m[k*simT+t+1]>mC){
                  x[k*simT+t+1]=x[k*simT+t+1]+extra_m(m[k*simT+t+1],age,t_il_t,mC,E,lean_complete);
                  m[k*simT+t+1]=mC;
                }
              }
              if(test1>0 && test2<0){
                x[k*simT+t+1] = x_lac_t;
                float milk = ((x[k*simT+t]-grow(leant_day[age], meat_cal, u)-x_lac_t)*u - cfs[0] - crs[0] + foods[0])*E/me;
                m[k*simT+t+1] = blub_perc(milk,age,m[k*simT+t],me,Er,u,t_il_t,lean_complete,leant_day,tw,meat_cal);
                if(m[k*simT+t+1]>mC){
                  x[k*simT+t+1]=x[k*simT+t+1]+extra_m(m[k*simT+t+1],age,t_il_t,mC,E,lean_complete);
                  m[k*simT+t+1]=mC;
                }
              }
              if(test1>0 && test2>=0){
                x[k*simT+t+1] = x[k*simT+t]-grow(leant_day[age], meat_cal, u)+ (- cfs[0] - crs[0] + foods[0] - Mmax*me/E)/u;
                float milk = Mmax;
                m[k*simT+t+1] = blub_perc(milk,age,m[k*simT+t],me,Er,u,t_il_t,lean_complete,leant_day,tw,meat_cal);
                if(m[k*simT+t+1]>mC){
                  x[k*simT+t+1]=x[k*simT+t+1]+extra_m(m[k*simT+t+1],age,t_il_t,mC,E,lean_complete);
                  m[k*simT+t+1]=mC;
                }
              }
              if(x[k*simT+t+1]>C_t){x[k*simT+t+1]=C_t;} 
              if(m[k*simT+t+1]<=mc){
                state[k*simT+t+1]=1;
              }else{
                state[k*simT+t+1]=3;
              }
            } //close lactating
            
            if(state[k*simT+t]==2){ //if she is pregnant
              Rcpp::NumericVector cfs=cf(x[k*simT+t]-grow(leant_day[age], meat_cal, u)+Wc[dy], lean_complete[age], raamr_t, preys, w, beta_t, psi_t, tc, l[k*simT+t], age, for_hoursu);
              Rcpp::NumericVector crs=cr(x[k*simT+t]-grow(leant_day[age], meat_cal, u)+Wc[dy], lean_complete[age], preys, w, beta_t, psi_t, tc, l[k*simT+t], age, for_hoursu);
              Rcpp::NumericVector foods=food(preys, w, beta_t, psi_t, tc, l[k*simT+t], age, for_hoursu, rho, A);
              x[k*simT+t+1]=x[k*simT+t]-grow(leant_day[age], meat_cal, u)+(- cfs[0] - crs[0] - ag(dy,Wc_d, cost_kg, propg, Chg_tot) + foods[0])/u;  
              if(x[k*simT+t+1]>C_t){x[k*simT+t+1]=C_t;}               
              state[k*simT+t+1]=2;
            } //close pregnant
            
            if((state[k*simT+t]!=2) && (state[k*simT+t]!=3)){ //if she is neither pregnant nor lactating
              Rcpp::NumericVector cfs=cf(x[k*simT+t]-grow(leant_day[age], meat_cal, u), lean_complete[age], raamr_t, preys, w, beta_t, psi_t, tc, l[k*simT+t], age, for_hoursu);
              Rcpp::NumericVector crs=cr(x[k*simT+t]-grow(leant_day[age], meat_cal, u), lean_complete[age], preys, w, beta_t, psi_t, tc, l[k*simT+t], age, for_hoursu);
              Rcpp::NumericVector foods=food(preys, w, beta_t, psi_t, tc, l[k*simT+t], age, for_hoursu, rho, A);
              x[k*simT+t+1]=x[k*simT+t]-grow(leant_day[age], meat_cal, u)+(- cfs[0] - crs[0] + foods[0])/u; 
              if(x[k*simT+t+1]>C_t){x[k*simT+t+1]=C_t;}              
              state[k*simT+t+1]=state[k*simT+t];
            } //close other state
          } //close strategy 1
          
          //strategy 2
          if(st[k*simT+t]==2){                    //if strategy is to move forward
            consec[k]=0;                          //reset number of consecutive days in patch
            l[k*simT+t+1]=l[k*simT+t]+1;
            Rcpp::NumericVector lambda_f(3);
            for (int la=0; la<3; ++la){
              lambda_f[la]=lambda[env[k]*(locs+1) + 2*(locs+1)*la + 2*(locs+1)*3*dyn + l[k*simT+t+1]];  //lambda depends on location and time
              }
            i[k*simT+t+1]=oneMultinom(lambda_f);  //pick a patch type reached in the new location
            
            if (state[k*simT+t]==3){              //if she is lactating
              double test1 = x[k*simT+t]-grow(leant_day[age], meat_cal, u) - cot(x[k*simT+t]-grow(leant_day[age], meat_cal, u),lean_complete[age])*d*(x[k*simT+t]-grow(leant_day[age], meat_cal, u)+lean_complete[age])/u - x_lac_t;             //if this is >0, she will deliver milk
              double test2 =(x[k*simT+t]-grow(leant_day[age], meat_cal, u)-x_lac_t)*u - cot(x[k*simT+t]-grow(leant_day[age], meat_cal, u),lean_complete[age])*d*(x[k*simT+t]-grow(leant_day[age], meat_cal, u)+lean_complete[age]) - Mmax*me/E;  //if this is >0, she will deliver Mmax milk
              if(test1<=0){
                x[k*simT+t+1] = x[k*simT+t] - grow(leant_day[age], meat_cal, u) - cot(x[k*simT+t]-grow(leant_day[age], meat_cal, u),lean_complete[age])*d*(x[k*simT+t]-grow(leant_day[age], meat_cal, u)+lean_complete[age])/u;
                float milk = 0;
                m[k*simT+t+1] = blub_perc(milk,age,m[k*simT+t],me,Er,u,t_il_t,lean_complete,leant_day,tw,meat_cal);
                if(m[k*simT+t+1]>mC){
                  x[k*simT+t+1]=x[k*simT+t+1]+extra_m(m[k*simT+t+1],age,t_il_t,mC,E,lean_complete);
                  m[k*simT+t+1]=mC;
                }
              }
              if(test1>0 & test2<0){
                x[k*simT+t+1] = x_lac_t;
                float milk = ((x[k*simT+t] - grow(leant_day[age], meat_cal, u)-x_lac_t)*u - cot(x[k*simT+t] - grow(leant_day[age], meat_cal, u),lean_complete[age])*d*(x[k*simT+t] - grow(leant_day[age], meat_cal, u)+lean_complete[age]))*E/me;
                m[k*simT+t+1] = blub_perc(milk,age,m[k*simT+t],me,Er,u,t_il_t,lean_complete,leant_day,tw,meat_cal);
                if(m[k*simT+t+1]>mC){
                  x[k*simT+t+1]=x[k*simT+t+1]+extra_m(m[k*simT+t+1],age,t_il_t,mC,E,lean_complete);
                  m[k*simT+t+1]=mC;
                }
              }
              if(test1>0 & test2>=0){
                x[k*simT+t+1] = x[k*simT+t]-grow(leant_day[age], meat_cal, u)+ (- cot(x[k*simT+t] - grow(leant_day[age], meat_cal, u),lean_complete[age])*d*(x[k*simT+t] - grow(leant_day[age], meat_cal, u)+lean_complete[age]) - Mmax*me/E)/u;
                float milk = Mmax;
                m[k*simT+t+1] = blub_perc(milk,age,m[k*simT+t],me,Er,u,t_il_t,lean_complete,leant_day,tw,meat_cal);
                if(m[k*simT+t+1]>mC){
                  x[k*simT+t+1]=x[k*simT+t+1]+extra_m(m[k*simT+t+1],age,t_il_t,mC,E,lean_complete);
                  m[k*simT+t+1]=mC;
                }
              }
              if(x[k*simT+t+1]>C_t){x[k*simT+t+1]=C_t;} 
              if(m[k*simT+t+1]<=mc){
                state[k*simT+t+1]=1;
              }else{
                state[k*simT+t+1]=3;
              }            
            } //close lactating 
            
            if(state[k*simT+t]==2){ //if she is pregnant
              x[k*simT+t+1] = x[k*simT+t] - grow(leant_day[age], meat_cal, u) +(- cot(x[k*simT+t]-grow(leant_day[age], meat_cal, u)+Wc[dy],lean_complete[age])*d*(x[k*simT+t]-grow(leant_day[age], meat_cal, u)+lean_complete[age]+Wc[dy])-ag(dy,Wc_d, cost_kg, propg, Chg_tot))/u;
              if(x[k*simT+t+1]>C_t){x[k*simT+t+1]=C_t;} 
              state[k*simT+t+1]=2;
            } //close pregnant 
            
            if((state[k*simT+t]!=2) && (state[k*simT+t]!=3)){ //if she is neither pregnant nor lactating
              x[k*simT+t+1]=x[k*simT+t]-grow(leant_day[age], meat_cal, u)-cot(x[k*simT+t]-grow(leant_day[age], meat_cal, u),lean_complete[age])*d*(x[k*simT+t]-grow(leant_day[age], meat_cal, u)+lean_complete[age])/u; 
              if(x[k*simT+t+1]>C_t){x[k*simT+t+1]=C_t;}          
              state[k*simT+t+1]=state[k*simT+t];
            } //close other state 
          } //close strategy 2

          //strategy 3/4
          if((st[k*simT+t]==3)||((l[k*simT+t]==1)&&(st[k*simT+t]==4))){
            consec[k]=0;                                                  //reset number of consecutive days in patch
            if((l[k*simT+t]==0)||((l[k*simT+t]==1)&&(st[k*simT+t]==4))){  //if in breeding ground
              l[k*simT+t+1]=l[k*simT+t];                                  //she moves within
            }else{
              l[k*simT+t+1]=l[k*simT+t]-1;                                //otherwise location decreases by 1
            }
            Rcpp::NumericVector lambda_f(3);
            for (int la=0; la<3; ++la){
              lambda_f[la]=lambda[env[k]*(locs+1) + 2*(locs+1)*la + 2*(locs+1)*3*dyn + l[k*simT+t+1]]; //lambda depends on location and time
              }
            i[k*simT+t+1]=oneMultinom(lambda_f);                          //pick a patch type reached in the new location
            
            if (state[k*simT+t]==3){ //if she is lactating
              double test1 = x[k*simT+t]-grow(leant_day[age], meat_cal, u) - cot(x[k*simT+t]-grow(leant_day[age], meat_cal, u),lean_complete[age])*d*(x[k*simT+t]-grow(leant_day[age], meat_cal, u)+lean_complete[age])/u - x_lac_t;             //if this is >0, she will deliver milk
              double test2 =(x[k*simT+t]-grow(leant_day[age], meat_cal, u)-x_lac_t)*u - cot(x[k*simT+t]-grow(leant_day[age], meat_cal, u),lean_complete[age])*d*(x[k*simT+t]-grow(leant_day[age], meat_cal, u)+lean_complete[age]) - Mmax*me/E;  //if this is >0, she will deliver Mmax milk
              if(test1<=0){
                x[k*simT+t+1] = x[k*simT+t] - grow(leant_day[age], meat_cal, u) - cot(x[k*simT+t]-grow(leant_day[age], meat_cal, u),lean_complete[age])*d*(x[k*simT+t]-grow(leant_day[age], meat_cal, u)+lean_complete[age])/u;
                float milk = 0;
                m[k*simT+t+1] = blub_perc(milk,age,m[k*simT+t],me,Er,u,t_il_t,lean_complete,leant_day,tw,meat_cal);
                if(m[k*simT+t+1]>mC){
                  x[k*simT+t+1]=x[k*simT+t+1]+extra_m(m[k*simT+t+1],age,t_il_t,mC,E,lean_complete);
                  m[k*simT+t+1]=mC;
                }
              }
              if(test1>0 & test2<0){
                x[k*simT+t+1] = x_lac_t;
                float milk = ((x[k*simT+t] - grow(leant_day[age], meat_cal, u)-x_lac_t)*u - cot(x[k*simT+t] - grow(leant_day[age], meat_cal, u),lean_complete[age])*d*(x[k*simT+t] - grow(leant_day[age], meat_cal, u)+lean_complete[age]))*E/me;
                m[k*simT+t+1] = blub_perc(milk,age,m[k*simT+t],me,Er,u,t_il_t,lean_complete,leant_day,tw,meat_cal);
                if(m[k*simT+t+1]>mC){
                  x[k*simT+t+1]=x[k*simT+t+1]+extra_m(m[k*simT+t+1],age,t_il_t,mC,E,lean_complete);
                  m[k*simT+t+1]=mC;
                }
              }
              if(test1>0 & test2>=0){
                x[k*simT+t+1] = x[k*simT+t]-grow(leant_day[age], meat_cal, u)+ (- cot(x[k*simT+t] - grow(leant_day[age], meat_cal, u),lean_complete[age])*d*(x[k*simT+t] - grow(leant_day[age], meat_cal, u)+lean_complete[age]) - Mmax*me/E)/u;
                float milk = Mmax;
                m[k*simT+t+1] = blub_perc(milk,age,m[k*simT+t],me,Er,u,t_il_t,lean_complete,leant_day,tw,meat_cal);
                if(m[k*simT+t+1]>mC){
                  x[k*simT+t+1]=x[k*simT+t+1]+extra_m(m[k*simT+t+1],age,t_il_t,mC,E,lean_complete);
                  m[k*simT+t+1]=mC;
                }
              }
              if(x[k*simT+t+1]>C_t){x[k*simT+t+1]=C_t;}       
              if(m[k*simT+t+1]<=mc){
                state[k*simT+t+1]=1;
              }else{
                state[k*simT+t+1]=3;
              }  
            } //close lactating 
            
            if(state[k*simT+t]==2){ //if she is pregnant
              x[k*simT+t+1] = x[k*simT+t] - grow(leant_day[age], meat_cal, u) +(- cot(x[k*simT+t]-grow(leant_day[age], meat_cal, u)+Wc[dy],lean_complete[age])*d*(x[k*simT+t]-grow(leant_day[age], meat_cal, u)+lean_complete[age]+Wc[dy])-ag(dy,Wc_d, cost_kg, propg, Chg_tot))/u;
              if(x[k*simT+t+1]>C_t){x[k*simT+t+1]=C_t;}          
              state[k*simT+t+1]=2;
            } //close pregnant 
            
            if((state[k*simT+t]!=2) && (state[k*simT+t]!=3)){ //if she is neither pregnat nor lactating
              x[k*simT+t+1]=x[k*simT+t]-grow(leant_day[age], meat_cal, u)-cot(x[k*simT+t]-grow(leant_day[age], meat_cal, u),lean_complete[age])*d*(x[k*simT+t]-grow(leant_day[age], meat_cal, u)+lean_complete[age])/u;       
              if(x[k*simT+t+1]>C_t){x[k*simT+t+1]=C_t;}             
              state[k*simT+t+1]=state[k*simT+t];
            } //close other state
          } //close strategy 3/4
        } 
      } 
  } //close individual loop

  List ret;
  ret["x"] = x;
  ret["state"] = state;
  ret["l"] = l;
  ret["i"] = i;
  ret["st"] = st;
  ret["m"] = m;
  ret["store"] = store;
  ret["consec"] = consec;
  ret["consec_calves"] = consec_calves;
  ret["alive"] = alive;
  ret["death"] = death;
  ret["nsto"] = nsto;
  ret["calves"] = calves;
  ret["agein"] = agein;
  ret["x_calves"] = x_calves;
  ret["l_calves"] = l_calves;
  ret["i_calves"] = i_calves;
  ret["st_calves"] = st_calves;
  ret["exposed"] = exposed;
  ret["resp"] = resp;
  ret["exposed_calves"] = exposed_calves;
  ret["resp_calves"] = resp_calves;
  ret["responsive_calves"] = responsive_calves;
  return ret;
}
