
#include <Rcpp.h>
#include <math.h>

using namespace std;
using namespace Rcpp;

//Ancillary functions

std::vector<int> which_equal(NumericVector x, float args){
  std::vector<int> ind(x.size());
  for (int i=0; i<ind.size(); ++i){
    if(x[i]==args) {ind[i]=1;}else{ind[i]=0;}
  }
  std::vector<int> out(ind);
  std::vector<int>::iterator it;
  int j = 0;
  it = std::find(ind.begin(), ind.end(), 1);
  while(it++ != ind.end()){
    out[j++] = it - ind.begin();
    it = std::find(it, ind.end(), 1);
  }
  out.resize(j);
  return out;
}

bool contains(NumericVector X, double z) {
  return std::find(X.begin(), X.end(), z)!=X.end();
}

int vecmax(NumericVector x) {    
  NumericVector::iterator it = std::max_element(x.begin(), x.end());
  return it - x.begin();
}

int vecmin(NumericVector x) {    
  NumericVector::iterator it = std::min_element(x.begin(), x.end());
  return it - x.begin();
}

int vec2max(NumericVector x) {
  NumericVector sorted = clone(x).sort();
  IntegerVector matched= match(sorted, x);
  return matched[matched.size()-2];
}

float roundTo2dec(float x)
{
  NumericVector vec(1);
  vec[0]=x;
  vec=round(vec,2);
  return vec[0];
}

float roundToThousandths(float x)
{
  x /=1000;
  return floor(x + 0.5) * 1000;
}

float roundToFiveHundreds(float x)
{
  x /=500;
  return floor(x + 0.5) * 500;
}

float roundToInteger(float x, int y)
{
  x /=y;
  return floor(x + 0.5) * y;
}

float roundToHundreds(float x)
{
  x /=100;
  return floor(x + 0.5) * 100;
}

NumericVector sequence(float minx, float maxx, float intx)
{
  int n=(maxx-minx)/intx;
  NumericVector x(n+1);
  double total=minx;
  x[0]=total;
  for(int i = 1; i < (n+1); ++i) {
    total = total+intx;
    x[i] = total;
  }
  return x;
}

NumericVector sequence_inv(float maxx, float minx, float intx)
{
  int n=round((maxx-minx)/intx);
  NumericVector x(n+1);
  double total=maxx;
  x[0]=total;
  for(int i = 1; i < (n+1); ++i) {
    total = total-intx;
    x[i] = total;
  }
  return x;
}

//Functions for state dynamics

float grow(float leant_dayt, float meat_cal, float u)  //cost of growth on a given day
{
  return leant_dayt*meat_cal/u;
}

float ag(int gtime, NumericVector Wc_d, float cost_kg, NumericVector propg, float Chg_tot)   //gestational costs given stage of pregnancy in days
{
  return Wc_d[gtime]*cost_kg + propg[gtime]*Chg_tot;
}

float cot(float blubber, float leant) //cost of transport given total weight
{
  float cost=7.79*pow(blubber + leant,-0.29);
  return cost;
}

float blub_perc(float milk, int t, float blub_per, float me, float Er, float u, int t_il_t, NumericVector lean_complete, NumericVector leant_day, int tw, float meat_cal)  //new calf condition (proportion of mass that is blubber), given milk received, previous condition, and time 
{
  int day=t-t_il_t;
  float bm=lean_complete[day]*blub_per/(1-blub_per); //current blubber mass, derived from %blubber and lean mass in current time step
  float Ed=milk*me;                                  //energy received by the calf, given amount of milk
  float ErT, MRj;
  MRj=2*(4*(pow(bm + lean_complete[day],0.75))*(24*60*60)/1000); //metabolic rate of the calf, given size
  ErT=MRj+leant_day[day]*meat_cal;                   //energy required on that day for maintenance and growth
  float S=Ed-ErT;                                    //balance of energy - it can be negative
  float D=S/u;                                       //kg of blubber that are deposited/used as the result of energy balance
  float nbm=D+bm;                                    //new blubber mass
  float perc=nbm/(lean_complete[day+1]+nbm);         //new condition 
  return perc;            
  
}

float extra_m(float nper, int t, int t_il_t, float mC, float E, NumericVector lean_complete)   //how much blubber mass the calf would receive above the maximum % blubber (so that it is 'given back' to the mother, since this won't be wasted into milk delivery)
{
  float pMax=lean_complete[t-t_il_t+1]*mC/(1-mC);      //maximum blubber mass the calf can have on the following day
  float extra;
  float nbm=lean_complete[t-t_il_t+1]*nper/(1-nper);   //blubber mass it would achieve based on the milk the mother can give
  if (nper>mC){
    extra=(nbm-pMax);
  }else{
    extra=0;
  }
  extra=extra/E;
  return extra; //extra mass (not delivered to the calf but retained by the female)
}

NumericVector lr(NumericVector dens, NumericVector w)   //lunging rate given krill density
{
  NumericVector wa(dens.size());
  LogicalVector lll=(dens<0.2);                 
  LogicalVector mmm=(dens>=0.2 & dens<=0.4);
  LogicalVector hhh=(dens>0.4);
  for (int i=0; i<dens.size(); i++){
    if(lll[i]) wa[i]=w[0];                 
    if(mmm[i]) wa[i]=w[1];
    if(hhh[i]) wa[i]=w[2];
  }
  return wa;
}

NumericVector tf(NumericVector dens, NumericVector w, float beta_t, float psi_t, int tc, int l_i, int t, float for_hoursu)      //time spent feeding (given patch density and anatomical limitations at given length)
{
  NumericVector ti(dens.size());
  NumericVector ni(dens.size());
  NumericVector lunge=lr(dens,w);
  NumericVector hours(dens.size());
  NumericVector frate(dens.size());
  for (int i=0; i<dens.size(); i++){
    frate[i]=lunge[i]*beta_t*dens[i];                      //max amount of food per hour
    if(frate[i]!=0) ti[i]=psi_t/frate[i];                  //time (hours) to fill forestomach (depends on density)
    if((ti[i]>tc) && (ti[i]!=0)) ni[i]=for_hoursu/ti[i];   //number of times it can fill the forestomach in a 12 hour period
    if((ti[i]<=tc) && (ti[i]!=0)) ni[i]=for_hoursu/tc;
    hours[i]=ti[i]*ni[i];
  }
  return hours;
}

NumericVector cf(float blubber, float leant, float raamr_t, NumericVector dens, NumericVector w, float beta_t, float psi_t, int tc, int l_i, int t, float for_hoursu) //cost of feeding 
{
  NumericVector cfeed(dens.size());
  NumericVector hours=tf(dens,w,beta_t,psi_t,tc,l_i,t,for_hoursu);
  for (int i=0; i<dens.size(); i++){
    cfeed[i]=hours[i]*60*60*raamr_t*(blubber+leant)/1000; 
  }
  return cfeed;
}

NumericVector cr(float blubber, float leant, NumericVector dens, NumericVector w, float beta_t, float psi_t, int tc, int l_i, int t, float for_hoursu) //cost of resting (given patch density to calculate time spent foraging, weight and anatomical limitations at given length)
{
  float BMR=4*(pow(blubber + leant,0.75))*(24*60*60)/1000;  
  float BMRr=2*BMR;                                     
  NumericVector crest(dens.size());
  NumericVector hours=tf(dens,w,beta_t,psi_t,tc,l_i,t,for_hoursu);
  for (int i=0; i<dens.size(); i++){
    crest[i]=(24-hours[i])/24*BMRr;
  }
  return crest;
}

NumericVector food(NumericVector dens, NumericVector w, float beta_t, float psi_t, int tc, int l_i, int t, float for_hoursu, int rho, float A)  //amount of energy from engulfed food (given patch density and anatomical limitations at given length)
{
  NumericVector ti(dens.size());
  NumericVector ni(dens.size());
  NumericVector lunge=lr(dens,w);
  NumericVector fq(dens.size());
  NumericVector frate(dens.size());
  for (int i=0; i<dens.size(); i++){
    frate[i]=lunge[i]*beta_t*dens[i];                      //max amount of food per hour
    if(frate[i]!=0) ti[i]=psi_t/frate[i];                  //time (hours) to fill forestomach (depends on density)
    if((ti[i]>tc) && (ti[i]!=0)) ni[i]=for_hoursu/ti[i];   //number of times it can fill the forestomach in a 12 hour period
    if((ti[i]<=tc) && (ti[i]!=0)) ni[i]=for_hoursu/tc;
    fq[i]=ni[i]*psi_t*rho*A;
  }
  return fq;
}

//Functions for fitness interpolations

int ClosestLower(NumericVector xs_wide, float x)  //function to find closest lower bin
{
  NumericVector maxs=xs_wide[xs_wide <= x];
  std::vector<int> cl= which_equal(xs_wide, max(maxs));
  return cl[0]-1; //-1 because used as index in C++
}


int Closest(NumericVector xs_wide, float x)  //function to find closest bin
{
  NumericVector diffs=abs(x-xs_wide);
  std::vector<int> cl= which_equal(xs_wide,xs_wide[vecmin(diffs)]);
  return cl[0]-1; //-1 because used as index in C++
}

NumericVector interp_mx(NumericVector x1, NumericVector m1, float mc, NumericVector xs_wide, NumericVector ms, int int_x, float int_m, int l_i, int i,NumericVector F1,int locs, int nstates, int e_i)  //interpolation for state 4
{
  NumericVector delta_x(x1.size());
  NumericVector delta_m(m1.size());
  NumericVector interpolm(x1.size());
  for(int j=0; j<x1.size(); j++){
    int x1t_i=ClosestLower(xs_wide,x1[j]);
    delta_x[j]=(x1[j]-xs_wide[x1t_i])/(xs_wide[x1t_i+1]-xs_wide[x1t_i]);
    
    int m1t_i=(m1[j]-mc)/int_m;
    float mi=(ms.size()-1)/(max(ms)-mc)*(m1[j]-mc);
    delta_m[j]=mi-m1t_i;
    
    interpolm[j]=(1-delta_x[j])*(1-delta_m[j])*F1[e_i*xs_wide.size()+2*xs_wide.size()*m1t_i+2*xs_wide.size()*ms.size()*l_i+2*xs_wide.size()*ms.size()*(locs+1)*i+2*xs_wide.size()*ms.size()*(locs+1)*3*3 + x1t_i] +
      (1-delta_x[j])*delta_m[j]*F1[e_i*xs_wide.size()+2*xs_wide.size()*(m1t_i+1)+2*xs_wide.size()*ms.size()*l_i+2*xs_wide.size()*ms.size()*(locs+1)*i+2*xs_wide.size()*ms.size()*(locs+1)*3*3 + x1t_i]+
      delta_x[j]*(1-delta_m[j])*F1[e_i*xs_wide.size()+2*xs_wide.size()*m1t_i+2*xs_wide.size()*ms.size()*l_i+2*xs_wide.size()*ms.size()*(locs+1)*i+2*xs_wide.size()*ms.size()*(locs+1)*3*3 +  x1t_i+1] +
      delta_x[j]*delta_m[j]*F1[e_i*xs_wide.size()+2*xs_wide.size()*(m1t_i+1)+2*xs_wide.size()*ms.size()*l_i+2*xs_wide.size()*ms.size()*(locs+1)*i+2*xs_wide.size()*ms.size()*(locs+1)*3*3 +  x1t_i+1];
  }
  return interpolm;
}

NumericVector interp_x(NumericVector x1, NumericVector xs_wide, NumericVector ms, int int_x, int l_i, int i,NumericVector F1,int locs, int nstates, int state, int e_i)  //interpolation for other states
{
  NumericVector delta_x(x1.size());
  NumericVector interpolm(x1.size());
  int m_tok=9; //subtract 1 because C++
  for(int j=0; j<x1.size(); j++){
    int x1t_i=ClosestLower(xs_wide,x1[j]);
    delta_x[j]=(x1[j]-xs_wide[x1t_i])/(xs_wide[x1t_i+1]-xs_wide[x1t_i]);
    
    interpolm[j]=(1-delta_x[j])*F1[e_i*xs_wide.size()+2*xs_wide.size()*m_tok+2*xs_wide.size()*ms.size()*l_i+2*xs_wide.size()*ms.size()*(locs+1)*i+2*xs_wide.size()*ms.size()*(locs+1)*3*state + x1t_i] +
      delta_x[j]*F1[e_i*xs_wide.size()+2*xs_wide.size()*m_tok+2*xs_wide.size()*ms.size()*l_i+2*xs_wide.size()*ms.size()*(locs+1)*i+2*xs_wide.size()*ms.size()*(locs+1)*3*state + x1t_i + 1];
  }
  return interpolm;
}

int oneMultinom(Rcpp::NumericVector probs)
{
  int k = probs.size();
  Rcpp::IntegerVector ans(k);
  rmultinom(1, probs.begin(), k, ans.begin());
  for(int p=0; p<ans.size(); ++p){
    if(ans[p]!=0) return p;
  }
}

//SDP function

// [[Rcpp::export]]

List fun_state2implant(NumericVector yj, NumericVector pj, NumericVector for_hours, NumericVector lambda, int a, int dd, NumericVector F1tmp, NumericVector F1, NumericVector resDl_n, NumericVector resRl_n, NumericVector resD2l_n, NumericVector resF0l_n)
{
  
  int T, locs, feed_g, int_x, d, Mmax, tc, rho, gamma, aM, N, maxA, t, n, dy, dyn, tb, ti, tw, ds, nstates, sizel, t_il_t, x_lac_t, indexl, int_low_x;
  double mc, mC, int_m, cost_kg, Chg_tot, me, E, wb, lb, ww, lw, lgd, meat_cal, MRc, LEd, u, raamr, beta, psi, A, m50, surv, Er, diff_l, intl, int_per, min_perc, death_perc, min_xc, max_C, xc_t, C_t, xc_tp, C_tp, raamr_t, beta_t, psi_t;
  NumericVector gtime, ms, Wc, Wc_d(334), propg(334), Chg(334), leand(212), w(3), ns, seqtbs, ts, per_lean, seq_length, percseq, xs_wide(25), xs_t, xs_tp, Ctmp, xctmp, x_lactmp;
  double calf_s, fet_s;
  
  aM=6;
  N=90;
  maxA=aM+N;
  ns=sequence(0,N,1); 
  NumericVector tbs(ns.size()), tis(ns.size()), tws(ns.size()), daysize(ns.size()), t_il(ns.size());
  t=dd+a*365-1;                     
  n=floor(((t+1)-(365*(aM-1)))/365)-1; 
  if(dd<32){
    dy=334+dd-1; 
  }else{
    dy=dd-31-1;
  }
  if(dy<(365-1)){    
    dyn=dy+1;
  }else{
    dyn=0;  
  }
  locs=36;   
  feed_g=15-1;
  tb=1-1;   
  ti=32-1;  
  tw=212-1; 
  ds=40-1; 
  T=tw+(aM+N)*365;  
  nstates=4;        
  ts=sequence(tw+1,T,1); 
  NumericVector lt(ts.size()), leant(ts.size());
  for(int i = 0; i < ns.size(); ++i) {
    tbs[i]=tb+(aM +ns[i])*365; 
    tis[i]=ti+(aM +ns[i])*365; 
    tws[i]=tw+(aM +ns[i])*365; 
  }
  if(t<tbs[0]){n=-1;}                    
  seqtbs=sequence_inv(T-1,tbs[tbs.size()-1],1);
  for(int i = 0; i < ns.size(); ++i) {
    t_il[i]=(tws[i]-1)-seqtbs.size()+1; 
  }
  surv=pow(0.975,(1.0/365.0));  
  calf_s=pow(0.875,1.0/212.0); 
  fet_s=pow(1-0.01,1.0/334.0);
  mc=0.05;
  mC=0.44;
  int_m=0.03;
  ms=sequence(mc,mC+int_m,int_m);
  d = 100;
  gtime=sequence(1,334,1);                   
  Wc=pow(0.52*(gtime-73),3.0)/1000.0;
  Wc[Wc<0.001]=0;
  cost_kg=2940*4.184; 
  for(int i = 0; i < (Wc.size()-1); ++i) {
    Wc_d[i]=(Wc[i+1]-Wc[i]);
    propg[i]=(Wc[i+1]-Wc[i])/max(Wc);
  }
  propg[Wc.size()-1]=NAN;
  Chg_tot=4400*4.184*pow(max(Wc),1.2); 
  Chg=propg*Chg_tot;                
  Mmax=220;     
  me=4137*4.184;
  E=0.9;
  wb=Wc[Wc.size()-1];
  lb=wb*0.94;
  ww=17000+wb;
  lw=ww*0.73; 
  lgd=(roundf(lw)-roundf(lb))/211; 
  leand[0]=roundf(lb);
  for(int i = 1; i < 212; ++i) {
    leand[i]= leand[i-1] + lgd; 
  }
  meat_cal=1500*4.184;
  LEd=lgd*meat_cal;
  MRc=200000*4.184;
  Er=MRc + LEd;
  u=31798;         
  w[0]=10; 
  w[1]=20;
  w[2]=30;
  tc=4;
  rho=4184;
  A=0.84;
  for(int i = 0; i < ts.size(); ++i) {
    lt[i]=22*(1-exp(-0.240*((ts[i]/365)+4.5)));
    leant[i]=0.73*4.6*pow(lt[i],3.05);
  }
  diff_l=(leant[2]-leand[leand.size()-1])/3.0;     
  leant[0]=leand[leand.size()-1]+diff_l*(1);
  leant[1]=leand[leand.size()-1]+diff_l*(2);
  leant[2]=leand[leand.size()-1]+diff_l*(3);
  sizel=leand.size()+leant.size();
  NumericVector lean_complete(sizel);
  for(int i = 0; i < lean_complete.size(); ++i) {
    if(i < (leand.size())){
      lean_complete[i]=leand[i];                        
    }else{
      indexl=i-leand.size();
      lean_complete[i]=leant[indexl];  
    }
  }
  NumericVector leant_day(lean_complete.size()-1), lt_prewean(leand.size()), lt_complete(lean_complete.size()), max_perc(lean_complete.size()), C(lean_complete.size()), xc(lean_complete.size()), x_lac(lean_complete.size());
  for(int i = 0; i < (lean_complete.size()-1); ++i) {
    leant_day[i]=roundf((lean_complete[i+1]-lean_complete[i]));
  }
  intl=(0.94-0.73)/(leand.size()-1);
  per_lean=sequence_inv(0.94,0.73,intl); 
  for(int i = 0; i < leand.size(); ++i) {
    lt_prewean[i]=pow(leand[i]/per_lean[i]/4.6,1/3.05);
  }
  for(int i = 0; i < lean_complete.size(); ++i) {
    if(i < leand.size()){
      lt_complete[i]=lt_prewean[i];                        
    }else{
      indexl=i-leand.size();
      lt_complete[i]=lt[indexl];  
    }
  }
  seq_length=sequence(tw+1,tbs[0],1);
  int_per=(0.44-0.35)/(seq_length.size()-1);
  percseq=sequence_inv(0.44,0.35,int_per);
  for(int i = 0; i < lean_complete.size(); ++i) {
    if(i < tw){
      max_perc[i]=0.44;                        
    }else{
      if (i < tbs[0]){
        max_perc[i]=percseq[i-tw];  
      }else{
        max_perc[i]=0.35;
      }
    }
  }
  min_perc=0.16; 
  death_perc=0.05;
  int_x=2000;
  int_low_x=500;
  min_xc=roundToInteger(death_perc*lean_complete[0]/(1-death_perc),int_low_x);        
  max_C=roundToInteger(max_perc[lean_complete.size()-1]*lean_complete[lean_complete.size()-1]/(1-max_perc[lean_complete.size()-1]),int_x); 
  xs_wide[sequence(0,8,1)]=sequence(0,4000,int_low_x);
  xs_wide[9]=4700;
  xs_wide[10]=5300;
  xs_wide[sequence(11,13,1)]=sequence(6000,8000,1000);
  xs_wide[14]=8700;
  xs_wide[15]=9300;
  xs_wide[sequence(16,18,1)]=sequence(10000,12000,1000);
  xs_wide[sequence(19,24,1)]=sequence(14000,24000,2000);
  Ctmp=max_perc*lean_complete/(1-max_perc); 
  x_lactmp=min_perc*lean_complete/(1-min_perc);
  xctmp=death_perc*lean_complete/(1-death_perc);
  for(int i = 0; i < lean_complete.size(); ++i) {
    C[i]=xs_wide[Closest(xs_wide,Ctmp[i])];
    xc[i]=xs_wide[Closest(xs_wide,xctmp[i])];
    x_lac[i]=xs_wide[Closest(xs_wide,x_lactmp[i])];
  }
  NumericVector bm(ms.size()-1), fms(ms.size()-1);
  gamma=8.0;
  m50=0.27;
  t_il_t=t_il[n];
  xc_t=xc[t];
  C_t=C[t]; 
  std::vector<int> xci=which_equal(xs_wide,xc_t);
  std::vector<int> Citmp=which_equal(xs_wide,C_t);
  int Ci=Citmp[0]+1;
  xs_t=xs_wide[sequence(xci[0]-1,Ci-1,1)];
  xc_tp=xc[t-1];
  C_tp=C[t-1];
  std::vector<int> xcpi=which_equal(xs_wide,xc_tp);
  std::vector<int> Cpitmp=which_equal(xs_wide,C_tp);
  int Cpi=Cpitmp[0]+1;
  xs_tp=xs_wide[sequence(xcpi[0]-1,Cpi-1,1)];
  x_lac_t=x_lac[t];
  raamr_t=26.23*pow(lt_complete[t],-0.809); 
  beta_t=0.0011*pow(lt_complete[t],3.56);  
  psi_t=0.47*pow(lt_complete[t],2.88);    
  int m_tok=9; 
  
  NumericVector F0(2*(xs_tp.size()-1)*(ms.size()-1)*(locs)*3), R(2*(xs_tp.size()-1)*(ms.size()-1)*(locs)*3), D(2*(xs_tp.size()-1)*(ms.size()-1)*(locs)*3), D2(2*(xs_tp.size()-1)*(ms.size()-1)*(locs)*3);
  
  for(int e_i=0; e_i<2; ++e_i){  
   for(int x_i=1; x_i<(xs_tp.size()-1); ++x_i){                    //cycle over all possible values of x (when x_i=0 she is dead)
    if(xs_tp[x_i] > xc_t){
      int m_i=m_tok;
      
      for (int l_i=0;l_i<1; ++l_i){                              //location 1 - Gulf of California (-1 for C++)
        NumericVector patches(2);
        patches[0]=0;
        patches[1]=2;
        for (int in=0; in < patches.size(); ++in){                               //cycle over other two patch types
          double x23,x22, for_hoursu, Db;
          NumericVector x13(30), x12(30), yju(30), pju(30), tmp2, tmp3, F113(30), F123(3),  x23v(1), lambda_f23(3), F133(3), lambda_f33(3), V3(3), F112(30), F122(3),  x22v(1), lambda_f22(3), F132(3), lambda_f32(3), V2(3), V(3), Vtmp(2), tmp4, F143(3), lambda_f43(3), F142(3), lambda_f42(3);
          int Dr;
          int i_i = patches[in];
          for (int yjj=0; yjj<30; ++yjj){
            yju[yjj]=yj[e_i*3 + 2*3*yjj + 2*3*30*l_i + 2*3*30*(locs+1)*dy + i_i];
            pju[yjj]=pj[e_i*3 + 2*3*yjj + 2*3*30*l_i + 2*3*30*(locs+1)*dy + i_i];
          }
          for_hoursu=for_hours[l_i*(365)+dy]; 
          
          //if she implants
          x13 = xs_tp[x_i]-grow(leant_day[t], meat_cal, u)+(-cf(xs_tp[x_i]-grow(leant_day[t], meat_cal, u)+ Wc[0], lean_complete[t], raamr_t, yju, w, beta_t, psi_t, tc, l_i, t, for_hoursu) - cr(xs_tp[x_i]-grow(leant_day[t], meat_cal, u)+ Wc[0], lean_complete[t], yju, w, beta_t, psi_t, tc, l_i, t, for_hoursu)- ag(0,Wc_d, cost_kg, propg, Chg_tot) + food(yju, w, beta_t, psi_t, tc, l_i, t, for_hoursu, rho, A))/u;
          for(int xx1=0; xx1<x13.size(); ++xx1){
            if(x13[xx1]<xc_t) x13[xx1]=xc_t;                         //truncated so that min is xc
            if(x13[xx1]>C_t) x13[xx1]=C_t;                           //truncated so that max is C
          }
          F113 = interp_x(x13,xs_wide,ms,int_x,l_i,i_i,F1,locs, nstates, 2, e_i);                 //calculates corresponding fitness
          
          x23 = xs_tp[x_i] - grow(leant_day[t], meat_cal, u)+ (- cot(xs_tp[x_i]-grow(leant_day[t], meat_cal, u)+ Wc[0],lean_complete[t])*d*(xs_tp[x_i]-grow(leant_day[t], meat_cal, u)+ Wc[0]+lean_complete[t])- ag(0,Wc_d, cost_kg, propg, Chg_tot))/u;
          if(x23<xc_t) x23=xc_t;                         //truncated so that min is xc
          if(x23>C_t) x23=C_t;                           //truncated so that max is C
          x23v[0]=x23;
          for (int iii=0;iii<3;++iii){
            tmp2= interp_x(x23v,xs_wide,ms,int_x,l_i+1,iii,F1,locs, nstates,2, e_i);
            F123[iii]=tmp2[0];
          }
          for (int la=0; la<3; ++la){
            lambda_f23[la]=lambda[e_i*(locs+1) + 2*(locs+1)*la + 2*(locs+1)*3*dyn + (l_i+1)];                 //lambda depends on location and time
          }
          
          for (int iii=0;iii<3;++iii){
            tmp3=interp_x(x23v,xs_wide,ms,int_x,l_i,iii,F1,locs,nstates,2, e_i);
            F133[iii]=tmp3[0];
          }
          for (int la=0; la<3; ++la){
            lambda_f33[la]=lambda[e_i*(locs+1) + 2*(locs+1)*la + 2*(locs+1)*3*dyn + (l_i)];                 //lambda depends on location and time
          }
          
          V3[0]=surv*sum(pju*F113);
          V3[1]=surv*sum(lambda_f23*F123);
          V3[2]=surv*sum(lambda_f33*F133);
          V3[0]=roundf(V3[0]*pow(10,14))/pow(10,14);
          V3[1]=roundf(V3[1]*pow(10,14))/pow(10,14);
          V3[2]=roundf(V3[2]*pow(10,14))/pow(10,14);
          
          //if she keeps resting
          x12 = xs_tp[x_i]-grow(leant_day[t], meat_cal, u)+(-cf(xs_tp[x_i]-grow(leant_day[t], meat_cal, u), lean_complete[t], raamr_t, yju, w, beta_t, psi_t, tc, l_i, t, for_hoursu) - cr(xs_tp[x_i]-grow(leant_day[t], meat_cal, u), lean_complete[t], yju, w, beta_t, psi_t, tc, l_i, t, for_hoursu) + food(yju, w, beta_t, psi_t, tc, l_i, t, for_hoursu, rho, A))/u;                    
          for(int xx1=0; xx1<x12.size(); ++xx1){
            if(x12[xx1]<xc_t) x12[xx1]=xc_t;                         //truncated so that min is xc
            if(x12[xx1]>C_t) x12[xx1]=C_t;                           //truncated so that max is C
          }
          F112 = interp_x(x12,xs_wide,ms,int_x,l_i,i_i,F1,locs, nstates, 1, e_i);                 //calculates corresponding fitness
          
          x22 = xs_tp[x_i] - grow(leant_day[t], meat_cal, u) - cot(xs_tp[x_i]-grow(leant_day[t], meat_cal, u),lean_complete[t])*d*(xs_tp[x_i]-grow(leant_day[t], meat_cal, u)+lean_complete[t])/u;                       
          if(x22<xc_t) x22=xc_t;                         //truncated so that min is xc
          if(x22>C_t) x22=C_t;                           //truncated so that max is C
          x22v[0]=x22;
          for (int iii=0;iii<3;++iii){
            tmp2= interp_x(x22v,xs_wide,ms,int_x,l_i+1,iii,F1,locs, nstates,1, e_i);
            F122[iii]=tmp2[0];
          }
          for (int la=0; la<3; ++la){
            lambda_f22[la]=lambda[e_i*(locs+1) + 2*(locs+1)*la + 2*(locs+1)*3*dyn + (l_i+1)];                 //lambda depends on location and time
          }
          
          for (int iii=0;iii<3;++iii){
            tmp3=interp_x(x22v,xs_wide,ms,int_x,l_i,iii,F1,locs,nstates,1, e_i);
            F132[iii]=tmp3[0];
          }
          for (int la=0; la<3; ++la){
            lambda_f32[la]=lambda[e_i*(locs+1) + 2*(locs+1)*la + 2*(locs+1)*3*dyn + (l_i)];                 //lambda depends on location and time
          }
          
          V2[0]=surv*sum(pju*F112);
          V2[1]=surv*sum(lambda_f22*F122);
          V2[2]=surv*sum(lambda_f32*F132);
          V2[0]=roundf(V2[0]*pow(10,14))/pow(10,14);
          V2[1]=roundf(V2[1]*pow(10,14))/pow(10,14);
          V2[2]=roundf(V2[2]*pow(10,14))/pow(10,14);
          
          //select action regarding reproduction
          if(max(V2)==max(V3)){
            Dr=5;
          }else{
            if(max(V2)>max(V3)){
              Dr=2;
            }else{
              Dr=3;
            }
          }
          
          //select action regarding behaviour
          if (Dr==2) {V[0]=V2[0]; V[1]=V2[1]; V[2]=V2[2];}
          if (Dr>2) {V[0]=V3[0]; V[1]=V3[1]; V[2]=V3[2];}
          F0[e_i*(xs_tp.size()-1)+2*(xs_tp.size()-1)*m_i+2*(xs_tp.size()-1)*(ms.size()-1)*l_i+2*(xs_tp.size()-1)*(ms.size()-1)*(locs)*i_i+x_i]=max(V);                                         //store max fitness
          if ((V[0]==V[1])&&(V[1]==V[2])){
            Db=4;
          }
          else
          {
            if((V[0]==V[1])&&(V[1]>V[2])){
              Db=5;
            }
            else
            {
              if((V[0]==V[2])&&(V[2]>V[1])){
                Db=6;
              }
              else
              {
                if((V[1]==V[2])&&(V[2]>V[0])){
                  Db=7;
                }
                else
                {
                  Db=(vecmax(V)+1);   //need to add 1 because its C++
                }
              }
            }
          }
          
          R[e_i*(xs_tp.size()-1)+2*(xs_tp.size()-1)*m_i+2*(xs_tp.size()-1)*(ms.size()-1)*l_i+2*(xs_tp.size()-1)*(ms.size()-1)*(locs)*i_i+x_i]=Dr;
          D[e_i*(xs_tp.size()-1)+2*(xs_tp.size()-1)*m_i+2*(xs_tp.size()-1)*(ms.size()-1)*l_i+2*(xs_tp.size()-1)*(ms.size()-1)*(locs)*i_i+x_i]=Db;
          D2[e_i*(xs_tp.size()-1)+2*(xs_tp.size()-1)*m_i+2*(xs_tp.size()-1)*(ms.size()-1)*l_i+2*(xs_tp.size()-1)*(ms.size()-1)*(locs)*i_i+x_i]=vec2max(V);   //no need to add 1 because this function operates in R terms
        }
      }
      
      for (int l_i=1;l_i<2; ++l_i){                              //location 2 (-1 for C++)
        NumericVector patches(2);
        patches[0]=0;
        patches[1]=2;
        for (int in=0; in < patches.size(); ++in){                               //cycle over other two patch types
          double x23,x22, for_hoursu, Db;
          NumericVector x13(30), x12(30), yju(30), pju(30), tmp2, tmp3, F113(30), F123(3),  x23v(1), lambda_f23(3), F133(3), lambda_f33(3), V3(4), F112(30), F122(3),  x22v(1), lambda_f22(3), F132(3), lambda_f32(3), V2(4), V(4), Vtmp(2), tmp4, F143(3), lambda_f43(3), F142(3), lambda_f42(3);
          int Dr;
          int i_i = patches[in];
          for (int yjj=0; yjj<30; ++yjj){
            yju[yjj]=yj[e_i*3 + 2*3*yjj + 2*3*30*l_i + 2*3*30*(locs+1)*dy + i_i];
            pju[yjj]=pj[e_i*3 + 2*3*yjj + 2*3*30*l_i + 2*3*30*(locs+1)*dy + i_i];
          }
          for_hoursu=for_hours[l_i*(365)+dy];
          
          //if she implants
          x13 = xs_tp[x_i]-grow(leant_day[t], meat_cal, u)+(-cf(xs_tp[x_i]-grow(leant_day[t], meat_cal, u)+ Wc[0], lean_complete[t], raamr_t, yju, w, beta_t, psi_t, tc, l_i, t, for_hoursu) - cr(xs_tp[x_i]-grow(leant_day[t], meat_cal, u)+ Wc[0], lean_complete[t], yju, w, beta_t, psi_t, tc, l_i, t, for_hoursu)- ag(0,Wc_d, cost_kg, propg, Chg_tot) + food(yju, w, beta_t, psi_t, tc, l_i, t, for_hoursu, rho, A))/u;
          for(int xx1=0; xx1<x13.size(); ++xx1){
            if(x13[xx1]<xc_t) x13[xx1]=xc_t;                         //truncated so that min is xc
            if(x13[xx1]>C_t) x13[xx1]=C_t;                           //truncated so that max is C
          }
          F113 = interp_x(x13,xs_wide,ms,int_x,l_i,i_i,F1,locs, nstates, 2, e_i);                 //calculates corresponding fitness
          
          x23 = xs_tp[x_i] - grow(leant_day[t], meat_cal, u)+ (- cot(xs_tp[x_i]-grow(leant_day[t], meat_cal, u)+ Wc[0],lean_complete[t])*d*(xs_tp[x_i]-grow(leant_day[t], meat_cal, u)+ Wc[0]+lean_complete[t])- ag(0,Wc_d, cost_kg, propg, Chg_tot))/u;
          if(x23<xc_t) x23=xc_t;                         //truncated so that min is xc
          if(x23>C_t) x23=C_t;                           //truncated so that max is C
          x23v[0]=x23;
          for (int iii=0;iii<3;++iii){
            tmp2= interp_x(x23v,xs_wide,ms,int_x,l_i+1,iii,F1,locs, nstates,2, e_i);
            F123[iii]=tmp2[0];
          }
          for (int la=0; la<3; ++la){
            lambda_f23[la]=lambda[e_i*(locs+1) + 2*(locs+1)*la + 2*(locs+1)*3*dyn + (l_i+1)];                 //lambda depends on location and time
          }
          
          for (int iii=0;iii<3;++iii){
            tmp3=interp_x(x23v,xs_wide,ms,int_x,l_i-1,iii,F1,locs,nstates,2, e_i);
            F133[iii]=tmp3[0];
          }
          for (int la=0; la<3; ++la){
            lambda_f33[la]=lambda[e_i*(locs+1) + 2*(locs+1)*la + 2*(locs+1)*3*dyn + (l_i-1)];                 //lambda depends on location and time
          }
          
          for (int iii=0;iii<3;++iii){
            tmp4=interp_x(x23v,xs_wide,ms,int_x,l_i,iii,F1,locs,nstates,2, e_i);
            F143[iii]=tmp4[0];
          }
          for (int la=0; la<3; ++la){
            lambda_f43[la]=lambda[e_i*(locs+1) + 2*(locs+1)*la + 2*(locs+1)*3*dyn + (l_i)];                 //lambda depends on location and time
          }
          
          V3[0]=surv*sum(pju*F113);
          V3[1]=surv*sum(lambda_f23*F123);
          V3[2]=surv*sum(lambda_f33*F133);
          V3[3]=surv*sum(lambda_f43*F143);
          V3[0]=roundf(V3[0]*pow(10,14))/pow(10,14);
          V3[1]=roundf(V3[1]*pow(10,14))/pow(10,14);
          V3[2]=roundf(V3[2]*pow(10,14))/pow(10,14);
          V3[3]=roundf(V3[3]*pow(10,14))/pow(10,14);
          
          //if she keeps resting
          x12 = xs_tp[x_i]-grow(leant_day[t], meat_cal, u)+(-cf(xs_tp[x_i]-grow(leant_day[t], meat_cal, u), lean_complete[t], raamr_t, yju, w, beta_t, psi_t, tc, l_i, t, for_hoursu) - cr(xs_tp[x_i]-grow(leant_day[t], meat_cal, u), lean_complete[t], yju, w, beta_t, psi_t, tc, l_i, t, for_hoursu) + food(yju, w, beta_t, psi_t, tc, l_i, t, for_hoursu, rho, A))/u;                    
          for(int xx1=0; xx1<x12.size(); ++xx1){
            if(x12[xx1]<xc_t) x12[xx1]=xc_t;                         //truncated so that min is xc
            if(x12[xx1]>C_t) x12[xx1]=C_t;                           //truncated so that max is C
          }
          F112 = interp_x(x12,xs_wide,ms,int_x,l_i,i_i,F1,locs, nstates, 1, e_i);                 //calculates corresponding fitness
          
          x22 = xs_tp[x_i] - grow(leant_day[t], meat_cal, u) - cot(xs_tp[x_i]-grow(leant_day[t], meat_cal, u),lean_complete[t])*d*(xs_tp[x_i]-grow(leant_day[t], meat_cal, u)+lean_complete[t])/u;                        
          if(x22<xc_t) x22=xc_t;                         //truncated so that min is xc
          if(x22>C_t) x22=C_t;                           //truncated so that max is C
          x22v[0]=x22;
          for (int iii=0;iii<3;++iii){
            tmp2= interp_x(x22v,xs_wide,ms,int_x,l_i+1,iii,F1,locs, nstates,1, e_i);
            F122[iii]=tmp2[0];
          }
          for (int la=0; la<3; ++la){
            lambda_f22[la]=lambda[e_i*(locs+1) + 2*(locs+1)*la + 2*(locs+1)*3*dyn + (l_i+1)];                 //lambda depends on location and time
          }
          
          for (int iii=0;iii<3;++iii){
            tmp3=interp_x(x22v,xs_wide,ms,int_x,l_i-1,iii,F1,locs,nstates,1, e_i);
            F132[iii]=tmp3[0];
          }
          for (int la=0; la<3; ++la){
            lambda_f32[la]=lambda[e_i*(locs+1) + 2*(locs+1)*la + 2*(locs+1)*3*dyn + (l_i-1)];                 //lambda depends on location and time
          }
          
          for (int iii=0;iii<3;++iii){
            tmp4=interp_x(x22v,xs_wide,ms,int_x,l_i,iii,F1,locs,nstates,1, e_i);
            F142[iii]=tmp4[0];
          }
          for (int la=0; la<3; ++la){
            lambda_f42[la]=lambda[e_i*(locs+1) + 2*(locs+1)*la + 2*(locs+1)*3*dyn + (l_i)];                 //lambda depends on location and time
          }
          
          V2[0]=surv*sum(pju*F112);
          V2[1]=surv*sum(lambda_f22*F122);
          V2[2]=surv*sum(lambda_f32*F132);
          V2[3]=surv*sum(lambda_f42*F142);
          V2[0]=roundf(V2[0]*pow(10,14))/pow(10,14);
          V2[1]=roundf(V2[1]*pow(10,14))/pow(10,14);
          V2[2]=roundf(V2[2]*pow(10,14))/pow(10,14);
          V2[3]=roundf(V2[3]*pow(10,14))/pow(10,14);
          
          //select action regarding reproduction
          if(max(V2)==max(V3)){
            Dr=5;
          }else{
            if(max(V2)>max(V3)){
              Dr=2;
            }else{
              Dr=3;
            }
          }
          
          //select action regarding behaviour
          if (Dr==2) {V[0]=V2[0]; V[1]=V2[1]; V[2]=V2[2]; V[3]=V2[3];}
          if (Dr>2) {V[0]=V3[0]; V[1]=V3[1]; V[2]=V3[2]; V[3]=V3[3];}
          F0[e_i*(xs_tp.size()-1)+2*(xs_tp.size()-1)*m_i+2*(xs_tp.size()-1)*(ms.size()-1)*l_i+2*(xs_tp.size()-1)*(ms.size()-1)*(locs)*i_i+x_i]=max(V);                                         //store max fitness
          if ((V[0]==V[1])&&(V[1]==V[2])&&(V[2]==V[3])){
            Db=5;
          }
          else
          {
            if((V[0]==V[1])&&(V[1]==V[2])&&(V[2]>V[3])){
              Db=6;
            }
            else
            {
              if((V[0]==V[1])&&(V[1]==V[3])&&(V[3]>V[2])){
                Db=7;
              }
              else
              {
                if((V[0]==V[2])&&(V[2]==V[3])&&(V[3]>V[1])){
                  Db=8;
                }
                else
                {
                  if((V[1]==V[2])&&(V[2]==V[3])&&(V[3]>V[0])){
                    Db=9;
                  }
                  else
                  {
                    if((V[0]==V[2])&&(V[2]==max(V))&&(V[2]>V[3])){
                      Db=10;
                    }
                    else
                    {
                      if((V[0]==V[1])&&(V[1]==max(V))&&(V[1]>V[3])){
                        Db=11;
                      }
                      else
                      {
                        if((V[0]==V[3])&&(V[3]==max(V))&&(V[3]>V[1])){
                          Db=12;
                        }
                        else
                        {
                          if((V[1]==V[2])&&(V[2]==max(V))&&(V[2]>V[0])){
                            Db=13;
                          }
                          else
                          {
                            if((V[1]==V[3])&&(V[3]==max(V))&&(V[3]>V[0])){
                              Db=14;
                            }
                            else
                            {
                              if((V[2]==V[3])&&(V[3]==max(V))&&(V[3]>V[0])){
                                Db=15;
                              }
                              else
                              {
                                Db=(vecmax(V)+1);
                              }
                            }
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }
          R[e_i*(xs_tp.size()-1)+2*(xs_tp.size()-1)*m_i+2*(xs_tp.size()-1)*(ms.size()-1)*l_i+2*(xs_tp.size()-1)*(ms.size()-1)*(locs)*i_i+x_i]=Dr;
          D[e_i*(xs_tp.size()-1)+2*(xs_tp.size()-1)*m_i+2*(xs_tp.size()-1)*(ms.size()-1)*l_i+2*(xs_tp.size()-1)*(ms.size()-1)*(locs)*i_i+x_i]=Db;
          D2[e_i*(xs_tp.size()-1)+2*(xs_tp.size()-1)*m_i+2*(xs_tp.size()-1)*(ms.size()-1)*l_i+2*(xs_tp.size()-1)*(ms.size()-1)*(locs)*i_i+x_i]=vec2max(V);   //no need to add 1 because this function operates in R terms
        }
      }
      
      for (int l_i=2;l_i<locs; ++l_i){                              //cycle over all other locations
        NumericVector patches(2);
        patches[0]=0;
        patches[1]=2;
        for (int i_i=0; i_i < 3; ++i_i){                               //cycle over other two patch types
          double x23,x22, for_hoursu, Db;
          NumericVector x13(30), x12(30), yju(30), pju(30), tmp2, tmp3, F113(30), F123(3),  x23v(1), lambda_f23(3), F133(3), lambda_f33(3), V3(3), F112(30), F122(3),  x22v(1), lambda_f22(3), F132(3), lambda_f32(3), V2(3), V(3), Vtmp(2), tmp4, F143(3), lambda_f43(3), F142(3), lambda_f42(3);
          int Dr;
          for (int yjj=0; yjj<30; ++yjj){
            yju[yjj]=yj[e_i*3 + 2*3*yjj + 2*3*30*l_i + 2*3*30*(locs+1)*dy + i_i];
            pju[yjj]=pj[e_i*3 + 2*3*yjj + 2*3*30*l_i + 2*3*30*(locs+1)*dy + i_i];
          }
          for_hoursu=for_hours[l_i*(365)+dy]; 
          
          //she cannot implant outside the breeding ground, so she keeps resting
          x12 = xs_tp[x_i]-grow(leant_day[t], meat_cal, u)+(-cf(xs_tp[x_i]-grow(leant_day[t], meat_cal, u), lean_complete[t], raamr_t, yju, w, beta_t, psi_t, tc, l_i, t, for_hoursu) - cr(xs_tp[x_i]-grow(leant_day[t], meat_cal, u), lean_complete[t], yju, w, beta_t, psi_t, tc, l_i, t, for_hoursu) + food(yju, w, beta_t, psi_t, tc, l_i, t, for_hoursu, rho, A))/u;                   
          for(int xx1=0; xx1<x12.size(); ++xx1){
            if(x12[xx1]<xc_t) x12[xx1]=xc_t;                         //truncated so that min is xc
            if(x12[xx1]>C_t) x12[xx1]=C_t;                           //truncated so that max is C
          }
          F112 = interp_x(x12,xs_wide,ms,int_x,l_i,i_i,F1,locs, nstates, 1, e_i);                 //calculates corresponding fitness
          
          x22 = xs_tp[x_i] - grow(leant_day[t], meat_cal, u) - cot(xs_tp[x_i]-grow(leant_day[t], meat_cal, u),lean_complete[t])*d*(xs_tp[x_i]-grow(leant_day[t], meat_cal, u)+lean_complete[t])/u;                        
          if(x22<xc_t) x22=xc_t;                         //truncated so that min is xc
          if(x22>C_t) x22=C_t;                           //truncated so that max is C
          x22v[0]=x22;
          for (int iii=0;iii<3;++iii){
            tmp2=interp_x(x22v,xs_wide,ms,int_x,l_i+1,iii,F1,locs, nstates,1, e_i);
            F122[iii]=tmp2[0];
          }
          for (int la=0; la<3; ++la){
            lambda_f22[la]=lambda[e_i*(locs+1) + 2*(locs+1)*la + 2*(locs+1)*3*dyn + (l_i+1)];                 //lambda depends on location and time
          }
          
          for (int iii=0;iii<3;++iii){
            tmp3=interp_x(x22v,xs_wide,ms,int_x,l_i-1,iii,F1,locs,nstates,1, e_i);
            F132[iii]=tmp3[0];
          }
          for (int la=0; la<3; ++la){
            lambda_f32[la]=lambda[e_i*(locs+1) + 2*(locs+1)*la + 2*(locs+1)*3*dyn + (l_i-1)];                 //lambda depends on location and time
          }
          
          V2[0]=surv*sum(pju*F112);
          V2[1]=surv*sum(lambda_f22*F122);
          V2[2]=surv*sum(lambda_f32*F132);
          V2[0]=roundf(V2[0]*pow(10,14))/pow(10,14);
          V2[1]=roundf(V2[1]*pow(10,14))/pow(10,14);
          V2[2]=roundf(V2[2]*pow(10,14))/pow(10,14);
          
          //select action regarding reproduction
          Dr=2;
          //select action regarding behaviour
          V[0]=V2[0];
          V[1]=V2[1];
          V[2]=V2[2];
          F0[e_i*(xs_tp.size()-1)+2*(xs_tp.size()-1)*m_i+2*(xs_tp.size()-1)*(ms.size()-1)*l_i+2*(xs_tp.size()-1)*(ms.size()-1)*(locs)*i_i+x_i]=max(V);                                         //store max fitness
          if ((V[0]==V[1])&&(V[1]==V[2])){
            Db=4;
          }
          else
          {
            if((V[0]==V[1])&&(V[1]>V[2])){
              Db=5;
            }
            else
            {
              if((V[0]==V[2])&&(V[2]>V[1])){
                Db=6;
              }
              else
              {
                if((V[1]==V[2])&&(V[2]>V[0])){
                  Db=7;
                }
                else
                {
                  Db=(vecmax(V)+1);   //need to add 1 because its C++
                }
              }
            }
          }
          
          R[e_i*(xs_tp.size()-1)+2*(xs_tp.size()-1)*m_i+2*(xs_tp.size()-1)*(ms.size()-1)*l_i+2*(xs_tp.size()-1)*(ms.size()-1)*(locs)*i_i+x_i]=Dr;
          D[e_i*(xs_tp.size()-1)+2*(xs_tp.size()-1)*m_i+2*(xs_tp.size()-1)*(ms.size()-1)*l_i+2*(xs_tp.size()-1)*(ms.size()-1)*(locs)*i_i+x_i]=Db;
          D2[e_i*(xs_tp.size()-1)+2*(xs_tp.size()-1)*m_i+2*(xs_tp.size()-1)*(ms.size()-1)*l_i+2*(xs_tp.size()-1)*(ms.size()-1)*(locs)*i_i+x_i]=vec2max(V);   //no need to add 1 because this function operates in R terms
        }
      }// close l loop
      
    } //close if statement for xc
   }  //close x loop
  } //close env loop
  
  
  //update results vectors and F1tmp
  
  std::vector<int> tmps=which_equal(xs_wide,xc_tp);
  int start=tmps[0];
  std::vector<int> tmpe=which_equal(xs_wide,C_tp);
  int end=tmpe[0];
  NumericVector seqx=sequence(start-1,end-1,1);    //-1 because c++ and used as index
  
  for (int ee=0; ee<2; ++ee){
   for (int xx=0; xx<(xs_tp.size()-1); ++xx){
    int xxxx=seqx[xx];
    for (int mm=0; mm<(ms.size()-1); ++mm){
      for (int ll=0; ll<locs; ++ll){
        for (int ii=0; ii<3; ++ii){
          F1tmp[ee*xs_wide.size()+2*xs_wide.size()*mm+2*xs_wide.size()*ms.size()*ll+2*xs_wide.size()*ms.size()*(locs+1)*ii+2*xs_wide.size()*ms.size()*(locs+1)*3*1+xxxx]=F0[ee*(xs_tp.size()-1)+2*(xs_tp.size()-1)*mm+2*(xs_tp.size()-1)*(ms.size()-1)*ll+2*(xs_tp.size()-1)*(ms.size()-1)*(locs)*ii+xx]; //the 1 here stands for the state (state 2, in C++ index 1)
          resDl_n[ee*xs_wide.size()+2*xs_wide.size()*mm+2*xs_wide.size()*ms.size()*ll+2*xs_wide.size()*ms.size()*(locs)*ii+2*xs_wide.size()*ms.size()*(locs)*3*1+2*xs_wide.size()*ms.size()*(locs)*3*nstates*(dd-1)+xxxx] = D[ee*(xs_tp.size()-1)+2*(xs_tp.size()-1)*mm+2*(xs_tp.size()-1)*(ms.size()-1)*ll+2*(xs_tp.size()-1)*(ms.size()-1)*(locs)*ii+xx];   //subtract 1 from dd for C++
          resRl_n[ee*xs_wide.size()+2*xs_wide.size()*mm+2*xs_wide.size()*ms.size()*ll+2*xs_wide.size()*ms.size()*(locs)*ii+2*xs_wide.size()*ms.size()*(locs)*3*1+2*xs_wide.size()*ms.size()*(locs)*3*nstates*(dd-1)+xxxx] = R[ee*(xs_tp.size()-1)+2*(xs_tp.size()-1)*mm+2*(xs_tp.size()-1)*(ms.size()-1)*ll+2*(xs_tp.size()-1)*(ms.size()-1)*(locs)*ii+xx];   //subtract 1 from dd for C++
          resD2l_n[ee*xs_wide.size()+2*xs_wide.size()*mm+2*xs_wide.size()*ms.size()*ll+2*xs_wide.size()*ms.size()*(locs)*ii+2*xs_wide.size()*ms.size()*(locs)*3*1+2*xs_wide.size()*ms.size()*(locs)*3*nstates*(dd-1)+xxxx] = D2[ee*(xs_tp.size()-1)+2*(xs_tp.size()-1)*mm+2*(xs_tp.size()-1)*(ms.size()-1)*ll+2*(xs_tp.size()-1)*(ms.size()-1)*(locs)*ii+xx]; //subtract 1 from dd for C++
          resF0l_n[ee*xs_wide.size()+2*xs_wide.size()*mm+2*xs_wide.size()*ms.size()*ll+2*xs_wide.size()*ms.size()*(locs)*ii+2*xs_wide.size()*ms.size()*(locs)*3*1+2*xs_wide.size()*ms.size()*(locs)*3*nstates*(dd-1)+xxxx] = F0[ee*(xs_tp.size()-1)+2*(xs_tp.size()-1)*mm+2*(xs_tp.size()-1)*(ms.size()-1)*ll+2*(xs_tp.size()-1)*(ms.size()-1)*(locs)*ii+xx]; //subtract 1 from dd for C++
        }
      }
    }
   }
  }
  
  if (C_t!=C_tp) {
   for (int ee=0; ee<2; ++ee){
    std::vector<int> tmpx=which_equal(xs_wide,C_t);
    int xct=tmpx[0];
    for (int mm=0; mm<(ms.size()-1); ++mm){
      for (int ll=0; ll<locs; ++ll){
        for (int ii=0; ii<3; ++ii){
          F1tmp[ee*xs_wide.size()+2*xs_wide.size()*mm+2*xs_wide.size()*ms.size()*ll+2*xs_wide.size()*ms.size()*(locs+1)*ii+2*xs_wide.size()*ms.size()*(locs+1)*3*1+xct]=(-99);
        }
      }
    }
   }
  }
  
  List ret;
  ret["F1"] = F1tmp;
  ret["resDl_n"] = resDl_n;
  ret["resRl_n"] = resRl_n;
  ret["resD2l_n"] = resD2l_n;
  ret["resF0l_n"] = resF0l_n;
  return ret;
}
